from dash import Dash, dcc, html, Input, Output, State, ALL
from .components import *

import os

class Plot:
    # Settings
    chart_bg = '#1f2c56'

    # State
    graph_df = pd.DataFrame()
    sort_method_current = ''
    filter_list = []
    add_button_count = 0
    chart1_button_clicks = 0
    chart2_button_clicks = 0
    chart3_button_clicks = 0
    chart3_back_button_clicks = 0
    save_clicks = 0
    line_selected = -1
    button_list = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
    chart_type = 1
    relayoutData = []
    init = True
    save_path = ''
    initial_capital = 0
    return_in_capital = 0
    mdd = 0
    sharp = 0

    start_date_datetime = 0
    end_date_datetime = 0

    para_combination = {}
    save_path = 'save_path'
    net_profit_to_mdd = 0
    net_profit = 0
    mdd_dollar = 0
    mdd_pct = 0
    num_of_trade = 0
    num_of_win = 0
    return_on_capital = 0
    sharp = 0
    total_commission = 0
    year_list = []
    year_count = []
    year_win_count = []


    def __new__(self, all_para_combination, result_df, settings):
        # start_date, end_date and para_dict is the same for all combination, ie use the 1st one
        start_date = all_para_combination[0]['start_date']
        end_date = all_para_combination[0]['end_date']
        para_dict = all_para_combination[0]['para_dict']
        py_filename = all_para_combination[0]['py_filename']
        output_folder = all_para_combination[0]['output_folder']
        intraday = all_para_combination[0]['intraday']
        summary_mode = all_para_combination[0]['summary_mode']
        freq = all_para_combination[0]['freq']
        self.initial_capital = all_para_combination[0]['sec_profile']['initial_capital']

        # For chart3 initial date range
        self.start_date_datetime = datetime.datetime.strptime(start_date, '%Y-%m-%d')
        self.end_date_datetime = self.start_date_datetime + datetime.timedelta(days=10)



        chart_bg = self.chart_bg

        components = Components()

        start_date_year = datetime.datetime.strptime(start_date, '%Y-%m-%d').year
        end_date_year = datetime.datetime.strptime(end_date, '%Y-%m-%d').year
        self.year_list = list(range(start_date_year, end_date_year + 1))

        empty_line_chart = components.empty_line_chart()
        checkbox_div = components.update_checkbox_div(para_dict, result_df)
        col_df = result_df.columns.values.tolist()
        df_col = pd.DataFrame([['-----' for column in col_df]], columns=col_df)
        performance_matrix = components.update_performance_matrix(start_date, end_date, df_col.iloc[0].copy(), para_dict)
        filter_div = components.update_filter_div([])


        sort_method_dropdown  = components.sort_method_dropdown
        filter_dropdown = components.filter_dropdown
        filter_dropdown_disabled = components.filter_dropdown_disabled
        filter_input = components.filter_input
        filter_input_disabled = components.filter_input_disabled
        add_button_style = components.add_button_style
        add_button_style_disabled = components.add_button_style_disabled



        app = Dash(__name__, external_stylesheets=[dbc.themes.SUPERHERO], suppress_callback_exceptions=True)

        my_css_data = """body {
                  background-color: #1a2245;
                }
                        /* width */
                ::-webkit-scrollbar {
                  width: 10px !important;
                  display: block !important;
                }

                /* Track */
                ::-webkit-scrollbar-track {
                  background: #1f2c56 !important;
                  border-radius: 10px !important;
                  display: block !important;
                }


                /* Handle */
                ::-webkit-scrollbar-thumb {
                  background: #154360;
                  border-radius: 10px;
                }
                """
        innerHtmlText = "<style>%s</style>" % my_css_data

        app.layout = html.Div([

            dash_dangerously_set_inner_html.DangerouslySetInnerHTML(innerHtmlText),


            html.Div(style={'height': '10px', }),
            html.Div(
                dbc.Row([
                    # Left Column
                    dbc.Col(html.Div([
                        html.Div(style={'height': '15px', }),
                        html.Div('Sorting Method', style={'color': 'Cyan', 'margin-left': '15px', 'font-size': '15px'}),
                        html.Div(style={'height': '10px', }),
                        sort_method_dropdown,
                        html.Div(style={'height': '15px', }),
                        html.Div('Parameters', style={'color': 'Cyan', 'margin-left': '15px', 'font-size': '15px'}),
                        html.Div(id='checklist-container',children=checkbox_div),
                        html.Div(style={'height': '10px', }),
                        html.Div('Filters for', style={'color': 'Cyan', 'margin-left': '15px', 'font-size': '15px'}),
                        html.Div(style={'height': '10px', }),
                        html.Div(id='filter', children=filter_div,style={'padding': '0px 20px',
                                                                         'padding-right': '10px',
                                                                         'font-size': '12px'}),
                        dbc.Row([
                            dbc.Col(id='filter_dropdown_div',children=filter_dropdown,width=8),
                            dbc.Col(id='filter_input_div',children=filter_input,width=3),
                        ]),
                        html.Div(style={'height': '10px', }),
                        html.Div(id='add_button',children=html.Div([html.Div(style={'height': '5px'}),'Add Filter']),
                                 style=add_button_style),

                    ],style={'padding':'0px','border-radius':'5px','height': '704px',
                             'background-color':'rgba(0, 0, 0, 0)'})
                        ,style={'padding':'0','padding-left':'5px'}, width=2),

                    # Middle Column
                    dbc.Col(html.Div([

                        html.Div(id='performance_matrix', children=performance_matrix),

                    ], style={'padding': '5px',})
                        , style={'padding': '0', }, width=2),


                    # Right Column
                    dbc.Col(html.Div([
                        html.Div(style={'height': '5px', }),

                        html.Div(id='main_chart_area',
                            children=[html.Div(html.Img(),style={'height':'10px'}),

                                      dbc.Row([
                                          dbc.Col(html.Div(id='chart1_button', children='',
                                                           style={'text-align': 'left', 'cursor': 'pointer',
                                                                  'font-size': '14px'}), width=4),
                                          dbc.Col(html.Div(html.Img()), width=4),
                                          dbc.Col(html.Div(id='chart2_button', children=''), width=4),
                                      ], style={'margin': '0px 5px'}),
                                      html.Div(id='chart_area',
                                               children=dcc.Graph(id='line_chart', figure=empty_line_chart)),

                                      html.Div(style={'height': '10px', }),

                                     ],style={'padding':'5px','border-radius':'5px','background-color':chart_bg}),


                        html.Div(id='chart3_area',
                                 children=[html.Div(html.Img(), style={'height': '10px'}),

                                           dbc.Row([
                                               dbc.Col(html.Div(id='chart3_back_button', children='< Back',
                                                                style={'text-align': 'left', 'cursor': 'pointer',
                                                                       'font-size': '14px'}), width=4),
                                               dbc.Col(html.Div(html.Img()), width=4)

                                           ], style={'margin': '0px 5px'}),
                                           html.Div(id='chart3',
                                                    children=dcc.Graph(id='chart3_chart', figure=empty_line_chart)),

                                           html.Div(style={'height': '10px', }),

                                           ],
                                 style={'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg, 'display': 'none'}),

                        html.Div(id='chart3_dashboard',
                            children=[html.Div(style={'height': '5px', }),
                                  html.Div(children=dbc.Row([
                                                dbc.Col(width=1),
                                                dbc.Col(children=html.Div([
                                                    html.Div(style={'height': '0.5px'}),
                                                    dcc.DatePickerRange(
                                                        id='chart3_date',
                                                        style={'margin-left':'15px'},
                                                        month_format='MMM Do, YY',
                                                        end_date_placeholder_text='MMM Do, YY',
                                                        start_date=datetime.date(self.start_date_datetime.year, self.start_date_datetime.month, self.start_date_datetime.day),
                                                        end_date=datetime.date(self.end_date_datetime.year, self.end_date_datetime.month, self.end_date_datetime.day),)
                                                ]),width=6),
                                                dbc.Col(children=html.Div(id='chart3_button',
                                                                          children=html.Div([html.Div(style={'height': '11px'}),'Show Intra Day Candlestick Chart']),
                                                                          style={'margin': '0px', 'width': '300px', 'backgroundColor': 'blue', 'padding': '0px',
                                                                                    'border-radius': '2.5px', 'text-align': 'center', 'cursor': 'pointer',
                                                                                    'font-size': '18px', 'height':'48px'}
                                                                    ), width=3),
                                            ], style={'width':'650px'})
                                           ,style={'padding': '15px 5px', 'border-radius': '5px', 'background-color': chart_bg})
                                  ], style={'display': 'none'}),

                        html.Div(style={'height': '5px', }),

                        html.Div(id='hist_area', children=html.Div(),
                                 style={'display': 'none'}),

                    ]), style={'padding':'0'}, width=8),
                ])
            ),

        ], style={'width':'1500px','margin':'auto','padding':'0px 10px','color':'white'})


        @app.callback(
            Output('line_chart', 'figure'),
            Output('filter', 'children'),
            Output('filter_dropdown_div', 'children'),
            Output('filter_input_div', 'children'),
            Output('add_button', 'style'),
            Output('filter_input', 'value'),
            Output('chart1_button', 'children'),
            Output('chart2_button', 'children'),
            Output('hist_area', 'children'),
            Output('hist_area', 'style'),
            Output('main_chart_area', 'style'),
            Output('chart3_dashboard', 'style'),
            Output('chart3_area', 'style'),
            Output('chart3_date', 'start_date'),
            Output('chart3_date', 'end_date'),
            Output('chart3_chart', 'figure'),
            Input('sort_method', 'value'),
            Input({'type': 'para-checklist', 'index': ALL}, 'value'),
            State('filter_dropdown_div', 'children'),
            State('filter_input_div', 'children'),
            State('line_chart', 'figure'),
            State('filter', 'children'),
            Input('add_button', 'n_clicks'),
            Input('chart1_button', 'n_clicks'),
            Input('chart2_button', 'n_clicks'),
            Input('chart3_button', 'n_clicks'),
            Input('chart3_back_button', 'n_clicks'),
            State('add_button', 'style'),
            State('filter_name', 'value'),
            State('filter_input', 'value'),
            State('hist_area', 'children'),
            State('hist_area', 'style'),
            State('main_chart_area', 'style'),
            State('chart3_dashboard', 'style'),
            State('chart3_area', 'style'),
            State('chart3_date', 'start_date'),
            State('chart3_date', 'end_date'),
            State('chart3_chart', 'figure'),
            [Input('button_' + str(i), 'n_clicks') for i in range(10)],
        )
        def display_output(sort_method,para_checklist,filter_dropdown_div,filter_input_div,fig_line,\
                           filter_div,add_button_clicks, \
                           chart1_button_clicks,chart2_button_clicks, chart3_button_clicks, chart3_back_button_clicks, \
                           _add_button_style,filter_name,filter_input_value,\
                           hist_area,hist_style,main_chart_area_style,chart3_dashboard_style, chart3_area_style, \
                           chart3_start_str, chart3_end_str, chart3_chart, *vals):

            chart1_button_text = ''
            chart2_button_text = ''

            chart1_button_click = False

            if chart3_button_clicks:
                if chart3_button_clicks > self.chart3_button_clicks:
                    self.chart3_button_clicks = chart3_button_clicks

                    if (chart3_start_str and chart3_end_str):
                        chart3_start = datetime.datetime.strptime(chart3_start_str, '%Y-%m-%d')
                        chart3_end = datetime.datetime.strptime(chart3_end_str, '%Y-%m-%d')
                        detla = chart3_end - chart3_start

                        if (detla.days > 60) or (detla.days < 1):
                            print('CandleStick Chart Limit: 1 to 60 Days')
                        else:
                            para_combination = self.para_combination
                            hist_style = {'display': 'none'}

                            chart3_chart = components.generate_chart_3(para_combination, chart3_start, chart3_end, freq)

                            main_chart_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg, 'display': 'none'}
                            chart3_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg,}


                    return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
                           None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
                           main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
                           chart3_start_str, chart3_end_str, chart3_chart

            if chart3_back_button_clicks:
                if chart3_back_button_clicks > self.chart3_back_button_clicks:
                    self.chart3_back_button_clicks = chart3_back_button_clicks
                    chart1_button_text = '< Back'
                    hist_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg}
                    main_chart_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg,}
                    chart3_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg,
                                         'display': 'none'}


                    return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
                           None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
                           main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
                           chart3_start_str, chart3_end_str, chart3_chart


            if chart1_button_clicks:
                if chart1_button_clicks > self.chart1_button_clicks:
                    chart1_button_click = True
                    self.chart1_button_clicks = chart1_button_clicks
                    self. chart_type = 1
                    hist_style = {'display': 'none'}
                    chart3_dashboard_style = {'display': 'none'}

            if chart2_button_clicks:
                if chart2_button_clicks > self.chart2_button_clicks:
                    self.chart2_button_clicks = chart2_button_clicks
                    if self.chart_type == 2:
                        chart1_button_text = '< Back'
                        return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
                               None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
                               main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
                               chart3_start_str, chart3_end_str, chart3_chart
                    else:
                        if self.line_selected > -1:
                            para_combination = self.para_combination
                            line_colour = self.graph_df.iloc[self.line_selected].line_colour

                            if (intraday or summary_mode):
                                df, fig_line = components.generate_chart_2_summary(para_combination, line_colour)

                            else:
                                df, fig_line = components.generate_chart_2_full(para_combination, line_colour, settings)

                            self.chart_type = 2
                            chart1_button_text = '< Back'


                            # For Chart 3 date range
                            if intraday:
                                chart3_dashboard_style = {}

                                df_chart3 = df.loc[df['action'].isnull() == False].copy()
                                self.start_date_datetime = df_chart3.iloc[0].date - datetime.timedelta(days=1)
                                self.end_date_datetime = self.start_date_datetime + datetime.timedelta(days=10)

                                chart3_start_str = datetime.date(self.start_date_datetime.year, self.start_date_datetime.month,
                                              self.start_date_datetime.day)
                                chart3_end_str = datetime.date(self.end_date_datetime.year, self.end_date_datetime.month,
                                              self.end_date_datetime.day)


                            # Histograms
                            period = settings['histogram_period']
                            # First column of the 5 charts
                            title_list = [dbc.Col(width=1)]
                            pct_list = [dbc.Col(html.Div('pct_change', style={'font-size': '12px', 'margin-top': '55px',
                                                                              'margin-left': '45px',
                                                                              'transform': 'rotate(-90deg)'}), width=1)]
                            rise_list = [dbc.Col(html.Div('max_rise', style={'font-size': '12px', 'margin-top': '55px',
                                                                              'margin-left': '45px',
                                                                              'transform': 'rotate(-90deg)'}), width=1)]
                            fall_list = [dbc.Col(html.Div('max_fall', style={'font-size': '12px', 'margin-top': '55px',
                                                                              'margin-left': '45px',
                                                                              'transform': 'rotate(-90deg)'}), width=1)]


                            for p in period:
                                title_list.append(dbc.Col(f'{p} Days',style={'font-size': '12px', 'text-align': 'center'},width=2))
                                fig_pct, fig_rise, fig_fall = components.generate_histogram(df, p, 'backtest')
                                pct_list.append( dbc.Col(dcc.Graph(figure=fig_pct,config={'displayModeBar': False})
                                                         ,style={'padding':'0'},width=2))
                                rise_list.append(dbc.Col(dcc.Graph(figure=fig_rise,config={'displayModeBar': False})
                                                         ,style={'padding': '0'}, width=2))
                                fall_list.append(dbc.Col(dcc.Graph(figure=fig_fall,config={'displayModeBar': False})
                                                         ,style={'padding': '0'}, width=2))


                            title_list = html.Div(dbc.Row(title_list))
                            pct_list = html.Div(dbc.Row(pct_list))
                            rise_list = html.Div(dbc.Row(rise_list))
                            fall_list = html.Div(dbc.Row(fall_list))

                            hist_area = [html.Div(style={'height': '5px', }),
                                         title_list, pct_list, rise_list, fall_list,
                                         html.Div(style={'height': '5px', })]

                            hist_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg}

                            return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style,\
                                   None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
                                   main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
                                   chart3_start_str, chart3_end_str, chart3_chart


                        else:
                            pass    # No Line Selected


            ## Initialize After Refresh
            if not self.init:
                if not add_button_clicks:
                    if not chart1_button_click:
                        self.sort_method_current = sort_method
                        self.filter_list = []
                        self.add_button_count = 0
                        self.chart2_button_clicks = 0
                        self.chart3_button_clicks = 0
                        self.chart3_back_button_clicks = 0
                        self.save_clicks = 0
                        self.chart_type = 1
                        self.line_selected = -1
                        hist_area = []
                        hist_style = {'display': 'none'}
                        self.init = True
                        filter_dropdown_div = filter_dropdown
                        filter_input_div = filter_input


            ## Add Button Pressed
            if add_button_clicks:
                if add_button_clicks > self.add_button_count: ## Add Button Pressed
                    self.add_button_count = add_button_clicks
                    if filter_name:
                        if filter_input_value: ## Paremeter
                            if filter_name == 'exclude':
                                self.filter_list.append(['exclude', ' ', filter_input_value])
                            else:
                                self.filter_list.append([filter_name[0:-1], filter_name[-1], filter_input_value])
                            filter_div = components.update_filter_div(self.filter_list)
                            if len(self.filter_list) > 11:
                                filter_dropdown_div = filter_dropdown_disabled
                                filter_input_div = filter_input_disabled
                                _add_button_style = add_button_style_disabled
                            else:
                                filter_dropdown_div = filter_dropdown
                                filter_input_div = filter_input
                            self.init = False

            else:
                self.add_button_count = 0  # Necessary for initialization


            # Filter delete button pressed, remove one filter
            for i in range(len(vals)):
                if not vals[i] == self.button_list[i]:
                    self.filter_list.pop(i)
                    filter_div = components.update_filter_div(self.filter_list)
                    filter_dropdown_div = filter_dropdown
                    filter_input_div = filter_input
                    self.add_button_count =- 1  # Necessary for add button count
                    if len(self.filter_list) < 12:
                        filter_dropdown_div = filter_dropdown
                        filter_input_div = filter_input
                        _add_button_style = add_button_style
                    break


            # Sort Method Selected, generate Chart 1
            if sort_method:
                current_df = result_df.copy()

                # Filter according to the filer list
                if len(self.filter_list) > 0:
                    for element in self.filter_list:
                        # print(element)
                        if element[0] == 'exclude':
                            try: code = int(element[2])
                            except: code = element[2]
                            current_df = current_df.loc[current_df['code'] != code]
                        elif element[1] == '<':
                            current_df = current_df.loc[current_df[element[0]] < float(element[2])]
                        else:
                            current_df = current_df.loc[current_df[element[0]] > float(element[2])]



                current_df = current_df.reset_index(drop=True)




                para_key_list = list(para_dict)



                if len(para_checklist) > 0:

                    and_list = []
                    for i in range(len(para_key_list)):
                        key = para_key_list[i]
                        or_list = []

                        for element in para_checklist[i]: # -1 because code / para_dict index

                            if element == 'True':element = True
                            elif element == 'False':element = False
                            # if key == 'tolerance':

                            _list = current_df[key] == element
                            # print(_list)
                            or_list.append(_list)
                        or_list = np.logical_or.reduce(or_list)
                        # print(key, or_list)
                        and_list.append(or_list)
                    and_list = np.logical_and.reduce(and_list)

                    df_checked = current_df.loc[pd.DataFrame(and_list, columns=['check'])['check']].reset_index(
                        drop=True).copy()
                else:

                    df_checked = current_df.copy()

                # Sort
                graph_df = components.sort_method_df(sort_method, df_checked)
                self.graph_df = graph_df

                fig_line = components.generate_chart_1(graph_df, all_para_combination, intraday, summary_mode)

                # Disable Chart 2
                chart2_button_text = 'Strategy Analysis >'
                self.chart_type = 1
                hist_style = {'display': 'none'}


            return fig_line, filter_div, filter_dropdown_div,filter_input_div,_add_button_style, \
                   None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
                   main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
                   chart3_start_str, chart3_end_str, chart3_chart



        # Click on Chart 1 line
        @app.callback(
            Output(component_id='performance_matrix', component_property='children'),
            Output('chart2_button', 'style'),
            Input('line_chart', 'clickData'),
            State(component_id='performance_matrix', component_property='children'),
        )
        def update_matrix(clickData,performance_matrix):
            button_style = {'text-align': 'right','color':'grey','font-size':'14px'}
            self.init = False
            if clickData:
                if self.chart_type == 1:
                    i = clickData['points'][0]['curveNumber'] - 1
                    performance_matrix = components.update_performance_matrix(start_date, end_date,
                                                                              self.graph_df.iloc[i].copy(), para_dict)
                    self.line_selected = i
                    button_style = {'text-align': 'right','cursor': 'pointer','font-size':'14px'}

                    reference_index = self.graph_df.iloc[i].reference_index
                    df_all_para_combination = pd.DataFrame(all_para_combination)
                    para_combination = \
                    df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict(
                        'records')[0]
                    self.para_combination = para_combination
                    save_path = plotguy.generate_filepath(para_combination=para_combination)

                    self.save_clicks = 0 # Needed for save button click tracking

                    df = self.graph_df.iloc[i].copy()
                    self.save_path = save_path
                    self.net_profit_to_mdd = df['net_profit_to_mdd']
                    self.net_profit = df['net_profit']
                    self.mdd_dollar = df['mdd_dollar']
                    self.mdd_pct = df['mdd_pct']
                    self.num_of_trade = df['num_of_trade']
                    self.num_of_win = round(int(df['num_of_trade'])*float(df['win_rate'])/100)
                    self.return_on_capital = df['return_on_capital']
                    self.sharp = df['annualized_sr']
                    self.total_commission = df['total_commission']
                    year_count = []
                    year_win_count = []
                    for year in self.year_list:
                        if not df[f'{year}_win_rate'] == '--':
                            win_count = round(int(df[str(year)])*float(df[f'{year}_win_rate'])/100)
                        else:
                            win_count = 0
                        year_count.append(df[str(year)])
                        year_win_count.append(win_count)
                    self.year_count = year_count
                    self.year_win_count = year_win_count

            return performance_matrix, button_style


        # Save Button
        @app.callback(
            Output('save_status', "children"),
            Output('save_string', "children"),
            Output('save_button', "style"),
            Input('save_button', "n_clicks"),
        )
        def save_button(save_clicks):
            if not os.path.isfile('saved_strategies.csv'):
                columns = ['path','initial','net_profit_to_mdd','net_profit',
                                      'mdd_dollar','mdd_pct','num_of_trade','num_of_win',
                                      'return_on_capital','total_commission','sharp'
                                      ]
                for year in self.year_list:
                    columns.append(year)
                    columns.append(f'{year}_win_count')
                columns.append('para_combination')
                pd.DataFrame(columns=columns).to_csv('saved_strategies.csv', index=False)
            else:
                pass

            button_available = None
            save_status = ''
            button_string = ''
            button_style = {}

            save_path = self.save_path
            if not self.init:
                if not save_clicks: #

                    save_status = ''
                    button_string = '--'
                    button_style = {'font-size': '15px', 'margin-left': '20px', 'margin-right': '30px',
                                    'text-align': 'center', 'border-radius': '5px',
                                    'background-color': 'Grey', 'color': 'black',
                                    }

                    if self.line_selected > -1:
                        saved_df = pd.read_csv('saved_strategies.csv')
                        if not save_path in list(saved_df['path']):
                            button_available = True
                        else:
                            button_available = False


                else:
                    if save_clicks > self.save_clicks:
                        self.save_clicks = save_clicks

                        if self.line_selected > -1:
                            saved_df = pd.read_csv('saved_strategies.csv')

                            if not save_path in list(saved_df['path']):
                                df_dict = {'path': save_path,
                                           'para_combination':str(self.para_combination),
                                           'initial': self.initial_capital,
                                           'net_profit_to_mdd': self.net_profit_to_mdd,
                                           'net_profit': self.net_profit,
                                           'mdd_dollar':self.mdd_dollar,
                                           'mdd_pct':self.mdd_pct,
                                           'num_of_trade':self.num_of_trade,
                                           'num_of_win': self.num_of_win,
                                           'return_on_capital': self.return_on_capital,
                                           'sharp': self.sharp,
                                           'total_commission': self.total_commission,
                                           }
                                for i, year in enumerate(self.year_list):
                                    df_dict[str(year)] = self.year_count[i]
                                    df_dict[f'{year}_win_count'] = self.year_win_count[i]

                                saved_df = pd.concat([saved_df, pd.DataFrame(df_dict,index=[10])],
                                                     ignore_index=True)
                                button_available = False
                            else:
                                index = saved_df.index[saved_df['path'] == save_path].tolist()
                                saved_df = saved_df.drop(index=index)
                                button_available = True

                            saved_df.to_csv('saved_strategies.csv', index=False)

            if button_available is not None:
                if button_available:
                    save_status = ''
                    button_string = 'Save Curve'
                    button_style = {'font-size': '15px', 'margin-left': '20px', 'margin-right': '30px',
                                    'text-align': 'center', 'border-radius': '5px', 'cursor': 'pointer',
                                    'background-color': 'Yellow', 'color': 'black',
                                    }
                else:
                    save_status = '( Saved )'
                    button_string = 'Unsave Curve'
                    button_style = {'font-size': '15px', 'margin-left': '20px', 'margin-right': '30px',
                                    'text-align': 'center', 'border-radius': '5px', 'cursor': 'pointer',
                                    'background-color': 'Magenta', 'color': 'white',
                                    }



            return save_status, button_string, button_style



        return app


from dash import Dash, dcc, html, Input, Output, State, ALL
import dash_daq as daq
from .components import *
import time

class Aggregate:

    chart_bg = '#1f2c56'
    valid_colour = 'DeepSkyBlue'
    df_equity_curves = pd.DataFrame()
    df_ready = False

    def __new__(self):

        chart_bg = self.chart_bg
        components = Components()
        valid_colour = self.valid_colour

        empty_line_chart = components.empty_line_chart()

        self.df_equity_curves, checklist_div = components.aggregate_df()

        app = Dash(__name__, external_stylesheets=[dbc.themes.SUPERHERO], suppress_callback_exceptions=True)

        my_css_data = """body {
          background-color: #1a2245;
        }
                /* width */
        ::-webkit-scrollbar {
          width: 10px !important;
          display: block !important;
        }
        
        /* Track */
        ::-webkit-scrollbar-track {
          background: #1f2c56 !important;
          border-radius: 10px !important;
          display: block !important;
        }
        
        
        /* Handle */
        ::-webkit-scrollbar-thumb {
          background: #154360;
          border-radius: 10px;
        }
        """
        innerHtmlText = "<style>%s</style>" % my_css_data


        app.layout = html.Div([

            dash_dangerously_set_inner_html.DangerouslySetInnerHTML(innerHtmlText),

            dbc.Row([
                # Left Column
                dbc.Col(html.Div([

                    html.Div(style={'height': '10px', }),

                    html.Div(children=html.Div([
                        html.Div(style={'height': '10px', }),

                        html.Div('Equity Curves', style={'color': 'DeepSkyBlue', 'font-size': '17px'}),

                        html.Div(style={'height': '10px', }),

                        html.Div(id='checklist-container', children=checklist_div,
                                 style={
                                        'maxHeight': '625px',
                                        'minHeight': '625px',
                                        'overflow-y': 'scroll',
                                        'overflow-x': 'hidden',
                                        }),

                    ],), style={'padding': '5px 5px','padding-left':'15px', 'border-radius': '5px',
                                'font-size': '13px','background-color': chart_bg},
                    ),

                ],), style={'padding': '0', 'padding-left': '5px',}, width=3),

                # Right Column
                dbc.Col(html.Div([



                    html.Div(style={'height': '15px', }),

                    html.Div([

                        html.Div(style={'height': '10px', }),

                        html.Div(html.Div('Aggregate', id='aggregate_string', style={'color': valid_colour}),
                                 style={'vertical-align': 'top','margin-left': '20px',
                                        'position': 'relative', 'top': '0.08em',
                                        'display': 'inline-block'}),

                        html.Div(daq.BooleanSwitch(on=True, color=valid_colour, id='aggregate_boolean',
                                                   style={'height': '5px', }),
                                 style={'margin-left': '10px', 'display': 'inline-block'}),

                        html.Div(html.Div('Normalized', id='normalized_string', style={'color': 'Grey'}),
                                 style={'margin-left': '15px', 'vertical-align': 'top',
                                        'position': 'relative', 'top': '0.08em',
                                        'display': 'inline-block'}),

                        html.Div(daq.BooleanSwitch(on=False, color=valid_colour, id='normalized_boolean',
                                                   style={'height': '5px', }),
                                 style={'margin-left': '10px', 'display': 'inline-block'}),
                    ]),

                    # html.Div(style={'height': '10px', }),

                    html.Div(children=html.Div([

                        dbc.Row([
                            # dbc.Col(html.Div(style={'height': '10px', }),

                            dbc.Col([

                                html.Div(style={'height': '15px', }),

                                html.Div(id='performance_div',
                                         style={'padding-left': '10px', 'font-size': '14px'}),

                                html.Div(style={'height': '15px', }),

                            ],style={'padding':'0px', 'padding-left': '28px',},width=3),

                            dbc.Col([

                                html.Div(style={'height': '5px', }),

                                html.Div(id='chart_area',
                                         children=dcc.Graph(id='line_chart',
                                                            figure=empty_line_chart)),

                                html.Div(style={'height': '20px', }),

                            ],style={'padding':'0px','padding-right': '35px',},width=9),
                        ]),

                    ]), style={'padding': '0px','border-radius': '5px',
                               # 'background-color': chart_bg
                               }),

                ]), style={'padding': '0', 'padding-left': '5px'}, width=9),

            ]),

        ], style={'width': '1500px', 'margin': 'auto', 'padding': '0px', 'color': 'white'})


        # For initial curve list
        @app.callback(
            Output('checklist-container', 'children'),
            Input('checklist-container', 'children'),
        )
        def init_checklist(checklist_container):
            self.df_ready = False
            self.df_equity_curves, checklist_div = components.aggregate_df()
            self.df_ready = True

            return checklist_div


        @app.callback(
            Output('line_chart', 'figure'),
            Output('normalized_string', 'style'),
            Output('aggregate_string', 'style'),
            Output('performance_div', 'children'),
            Input('curve_checklist', 'value'),
            Input('normalized_boolean','on'),
            Input('aggregate_boolean', 'on')
        )
        def aggregate_chart(curve_checklist,normalized_boolean, aggregate_boolean):

            while not self.df_ready:
                time.sleep(0.2)

            curve_checklist.sort()
            valid_colour = self.valid_colour

            title_text = ''

            if aggregate_boolean:
                aggregate_style = {'color': valid_colour}
                title_text += 'Aggregate '
            else:
                aggregate_style = {'color': 'Grey'}

            if normalized_boolean:
                normalized_style = {'color': valid_colour}
                title_text += 'Normalized '
            else:
                normalized_style = {'color': 'Grey'}

            title_text += 'Equity Curves'


            df_curves_info = self.df_equity_curves.loc[curve_checklist]
            folders = list(df_curves_info['folder'])
            pys = list(df_curves_info['py'])
            paths = list(df_curves_info['path'])
            para_combination = list(df_curves_info['para_combination'])

            # Aggregate Equity
            dfs = []
            for i, path in enumerate(paths):
                intraday = para_combination[i]['intraday']
                summary = para_combination[i]['summary_mode']
                df_csv = pd.read_csv(path, index_col='date')
                if (intraday or summary):
                    df = plotguy.resample_summary_to_daily(para_combination=para_combination[i],folder=folders[i])
                else:
                    df = df_csv

                df_current = pd.DataFrame()
                if normalized_boolean:
                    df_current[f'{folders[i]}_{pys[i]}_{i}'] = df['equity_value'] / (df['equity_value'].iloc[0] / 100000)
                else:
                    df_current[f'{folders[i]}_{pys[i]}_{i}'] = df['equity_value']
                df_current.index = df.index
                dfs.append(df_current.copy())

            df_all_curves = pd.concat(dfs, axis=1).sort_index()
            df_all_curves = df_all_curves.fillna(method='pad')
            df_all_curves = df_all_curves.fillna(method='backfill')
            df_all_curves["Aggregate_Equity"] = df_all_curves.sum(axis=1)
            if normalized_boolean:
                df_all_curves["Aggregate_Equity"] = df_all_curves["Aggregate_Equity"] / (len(df_all_curves.columns) - 1)
            # print(df_all_curves)

            # Performance
            dicts = []
            for i, row in df_curves_info.iterrows():
                d = row['performance'].copy()

                if normalized_boolean:
                    initial_capital = d['initial_capital']
                    d['initial_capital'] = d['initial_capital'] / (initial_capital / 100000)
                    d['net_profit'] = d['net_profit'] / (initial_capital / 100000)
                    d['total_commission'] = d['total_commission'] / (initial_capital / 100000)
                d.pop('net_profit_to_mdd')
                d.pop('mdd_dollar')
                d.pop('mdd_pct')
                d.pop('return_on_capital')
                d.pop('sharp_ratio')
                dicts.append(d.copy())

            df_performance = pd.DataFrame.from_dict(dicts)

            total_dict = {}
            for element in df_performance.columns:
                total_dict[element] = df_performance[element].sum()


            total_dict['return_on_capital'] = total_dict['net_profit']/total_dict['initial_capital']
            total_dict['win_rate'] = total_dict['num_of_win']/total_dict['num_of_trade']



            df_agg = df_all_curves['Aggregate_Equity']
            dds = []
            dd_pct = []
            for i in range(len(df_agg)):
                max_list = list(df_agg[:i + 1]).copy()
                dd = max(max_list) - df_agg.iloc[i]
                dds.append(dd)
                dd_pct.append( dd / max(max_list))
                # print(dd, max(max_list), dd / max(max_list))
            mdd = max(dds)
            mdd_pct = max(dd_pct)

            total_dict['mdd_dollar'] = mdd
            total_dict['mdd_pct'] = mdd_pct
            total_dict['net_profit_to_mdd'] = total_dict['net_profit'] / total_dict['mdd_dollar']

            df_all_curves['date'] = pd.to_datetime(df_all_curves.index, format='%Y-%m-%d')
            holding_period_day = (df_all_curves.loc[df_all_curves.index[-1], 'date'] - df_all_curves.loc[
                df_all_curves.index[0], 'date']).days

            equity_value_pct_series = df_all_curves["Aggregate_Equity"].pct_change()
            equity_value_pct_series = equity_value_pct_series.dropna()

            return_on_capital = total_dict['return_on_capital']

            annualized_return = (1 + return_on_capital) ** (365 / holding_period_day) - 1
            annualized_std = equity_value_pct_series.std() * math.sqrt(365)
            if annualized_std > 0:
                annualized_sr = annualized_return / annualized_std
            else:
                annualized_sr = 0

            total_dict['annualized_return'] = annualized_return
            total_dict['annualized_std'] = annualized_std
            total_dict['annualized_sr'] = annualized_sr

            start_date_year = df_all_curves.loc[df_all_curves.index[0], 'date'].year
            end_date_year = df_all_curves.loc[df_all_curves.index[-1], 'date'].year
            year_list = list(range(start_date_year, end_date_year + 1))

            aggregate_performance = components.aggregate_performance(total_dict, year_list)


            # Generate Chart
            if aggregate_boolean:
                fig_line = px.line()
                fig_line.update_layout(title={'text': title_text })
                fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
                fig_line.update_yaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
                fig_line.update_layout(plot_bgcolor='#1a2245', paper_bgcolor='#1a2245', height=650,
                                       margin=dict(l=0, r=25, t=60, b=0),
                                       showlegend=True,
                                       font={"color": "white", 'size': 10.5}, yaxis={'title': ''},
                                       xaxis={'title': ''}
                                       )
                fig_line.add_trace(go.Scatter(mode='lines', # hovertemplate=hovertemplate,
                                              x=df_all_curves.index, y=df_all_curves['Aggregate_Equity'],
                                              line=dict(color=valid_colour, width=1.5), name='Aggregate'), )
            else:
                fig_line = px.line()
                fig_line.update_layout(title={'text': title_text})
                fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
                fig_line.update_yaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
                fig_line.update_layout(plot_bgcolor='#1a2245', paper_bgcolor='#1a2245', height=650,
                                       margin=dict(l=0, r=25, t=60, b=0),
                                       showlegend=True,
                                       font={"color": "white", 'size': 10.5}, yaxis={'title': ''},
                                       xaxis={'title': ''},
                                       )
                # print(list(df_all_curves.columns))

                columns = list(df_all_curves.columns)[:-1]
                for curve_number, column in zip(curve_checklist,columns):
                    fig_line.add_trace(go.Scatter(mode='lines',  # hovertemplate=hovertemplate,
                                                  x=df_all_curves.index, y=df_all_curves[column],
                                                  line=dict(color=self.df_equity_curves.loc[curve_number].line_colour, width=1.5), name=f'Curve {str(curve_number).zfill(3)}'), )



            return fig_line, normalized_style, aggregate_style, aggregate_performance





        return app


from dash import Dash, dcc, html, Input, Output, State, ALL
from .components import *

class Signals:
    # Settings
    chart_bg = '#1f2c56'

    def __new__(self, filename, output_folder, start_date, end_date, para_dict, generate_filepath, settings):

        chart_bg = self.chart_bg
        components = Components()
        empty_line_chart = components.empty_line_chart()
        radioitems_div = components.generate_radioitems(para_dict)
        period = settings['histogram_period']
        stat_ = ['---', '---', '---', '---', '---']
        para_values =  ['-----' for key in para_dict]

        app = Dash(__name__, external_stylesheets=[dbc.themes.SUPERHERO], suppress_callback_exceptions=True)

        my_css_data = """body {
                  background-color: #1a2245;
                }
                        /* width */
                ::-webkit-scrollbar {
                  width: 10px !important;
                  display: block !important;
                }

                /* Track */
                ::-webkit-scrollbar-track {
                  background: #1f2c56 !important;
                  border-radius: 10px !important;
                  display: block !important;
                }


                /* Handle */
                ::-webkit-scrollbar-thumb {
                  background: #154360;
                  border-radius: 10px;
                }
                """
        innerHtmlText = "<style>%s</style>" % my_css_data

        app.layout = html.Div([

            dash_dangerously_set_inner_html.DangerouslySetInnerHTML(innerHtmlText),

            html.Div(style={'height': '5px', }),

            html.Div(
                dbc.Row([
                    # Left Column
                    dbc.Col(html.Div([

                        html.Div(style={'height': '10px', }),

                        html.Div('Selected Parameters', style={'color': 'Cyan', 'font-size': '16px'}),

                        html.Div(id='title_area',
                                 children=components.selection_title(para_dict, para_values),
                                 style={'padding-left': '10px', 'font-size': '14px'}),

                        html.Div(style={'height': '20px', }),

                        html.Div(id='radioitems_container',children=radioitems_div,style={'padding-right':'30px'}),

                    ],style={'padding':'0px','border-radius':'5px',
                             'padding-left':'50px','padding-right':'15px','height': '704px',
                             'background-color':'rgba(0, 0, 0, 0)'})
                        ,style={'padding':'0',}, width=4),

                    # Right Column
                    dbc.Col(html.Div([

                        html.Div(style={'height': '10px', }),

                        html.Div([html.Div(html.Img(),style={'height':'5px'}),
                                  html.Div(id='chart_area',
                                           children=dcc.Graph(id='line_chart', figure=empty_line_chart)),

                                 ],style={'padding':'5px','border-radius':'5px',
                                          'padding-right': '25px',
                                          'background-color':chart_bg}),

                        html.Div(style={'height': '5px', }),


                        html.Div(children=html.Div([
                            html.Div(style={'height': '5px', }),

                            html.Div('Average', style={'color': 'Cyan', 'font-size': '14px'}),

                            html.Div(id='stat_area',
                                     children=components.update_stat_div(period, stat_, stat_, stat_),
                                     style={'padding': '0px 20px','text-align': 'center','font-size': '12px',}),

                            html.Div(style={'height': '5px', }),

                            html.Div('Histogram', style={'color': 'Cyan', 'font-size': '14px'}),


                            html.Div(id='hist_area', children=html.Div(),
                                     style={'padding': '0px 20px','text-align': 'center'}),

                        ]),style={'padding': '5px 25px', 'border-radius': '5px',
                                  'height': '380px', 'font-size': '13px',
                                   'background-color': chart_bg}),

                    ]), style={'padding':'0','padding-left':'5px'}, width=8),
                ])
            ),

        ], style={'width':'1200px','margin':'auto','padding':'0px','color':'white'})

        @app.callback(
            Output('title_area', 'children'),
            Output('stat_area', 'children'),
            Output('line_chart', 'figure'),
            Output('hist_area', 'children'),
            State('title_area', 'children'),
            State('stat_area', 'children'),
            State('line_chart', 'figure'),
            State('hist_area', 'children'),
            Input({'type': 'para_radioitems', 'index': ALL}, 'value'),
        )
        def display_output(title_area, stat_area, line_chart, hist_area, para_radioitems):

            # if not complete selection
            if None in para_radioitems:
                return title_area, stat_area, line_chart, hist_area

            save_path = generate_filepath(
                py_filename=filename,
                output_folder=output_folder,
                start_date=start_date,
                end_date=end_date,
                para_dict=para_dict,
                para_combination=para_radioitems)

            df = pd.read_csv(save_path)

            # Count number of subchart
            subchart_count = 0
            try:
                fig = settings['subchart_1']
                subchart_count += 1
            except Exception as e: pass
            try:
                fig = settings['subchart_2']
                subchart_count += 1
            except Exception as e: pass

            if subchart_count == 0:
                row_height = [1]
                height = 350
            if subchart_count == 1:
                row_height = [0.85, 0.15]
                height = 420
            if subchart_count == 2:
                row_height = [0.7, 0.15, 0.15]
                height = 500

            title = ''
            for i, key in enumerate(para_dict):
                title = title + f'{key}:{para_radioitems[i]} '

            # Close Chart
            df_chart = df.copy()

            open_col = []
            for i, row in df_chart.iterrows():
                if row['logic'] == 'trade_logic':
                    open_col.append(row['close'])
                else:
                    open_col.append(None)

            df_chart['open_signal'] = open_col

            # fig_line = go.Figure()
            fig_line = make_subplots(rows=subchart_count + 1, cols=1,
                                     row_heights=row_height, shared_xaxes=True)
            fig_line.update_layout(title={'text': title, 'font': {'size': 12}})
            fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
            fig_line.update_yaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
            fig_line.update_layout(plot_bgcolor=self.chart_bg, paper_bgcolor=self.chart_bg, height=height,
                                   margin=dict(l=85, r=80, t=35, b=20),
                                   font={"color": "white", 'size': 9}, yaxis={'title': 'Close'},
                                   xaxis={'title': ''})
            fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
                                          x=df_chart['date'], y=df_chart['close'],
                                          line=dict(color='#00FFFF', width=1), name='Close'),
                               row=1, col=1)
            fig_line.add_trace(go.Scatter(mode='markers',
                                          x=df_chart['date'], y=df_chart['open_signal'],
                                          marker=dict(color='rgba(0, 0, 0, 0)', size=9,
                                                      line=dict(color='yellow', width=2.5)), name='Signal'),
                               row=1, col=1)

            subchart_count = 2
            try:
                fig = components.generate_subchart(df_chart, settings['subchart_1'][0], settings['subchart_1'][1], 'yellow')
                fig_line.add_trace(fig, row=subchart_count, col=1)
                subchart_count += 1
            except Exception as e:
                # print(e)
                pass
            try:
                fig = components.generate_subchart(df_chart, settings['subchart_2'][0], settings['subchart_2'][1], '#FF01FE')
                fig_line.add_trace(fig, row=subchart_count, col=1)
            except Exception as e:
                # print(e)
                pass

            line_chart = fig_line


            title_list = [dbc.Col(width=2)]
            pct_list = [dbc.Col(html.Div('pct_change', style={'font-size': '12px'}), width=2)]
            rise_list = [dbc.Col(html.Div('max_rise', style={'font-size': '12px'}), width=2)]
            fall_list = [dbc.Col(html.Div('max_fall', style={'font-size': '12px'}), width=2)]
            pct_mean = []
            rise_mean = []
            fall_mean = []

            for p in period:
                title_list.append(dbc.Col(f'{p} Days',style={'font-size': '12px',
                                                             'text-align': 'center'},width=2))
                fig_pct, fig_rise, fig_fall = components.generate_histogram(df, p, 'signal')
                pct_list.append( dbc.Col(dcc.Graph(figure=fig_pct,config={'displayModeBar': False})
                                         ,style={'padding':'0'},width=2))
                rise_list.append(dbc.Col(dcc.Graph(figure=fig_rise,config={'displayModeBar': False})
                                         ,style={'padding': '0'}, width=2))
                fall_list.append(dbc.Col(dcc.Graph(figure=fig_fall,config={'displayModeBar': False})
                                         ,style={'padding': '0'}, width=2))

                stat_pct, stat_rise, stat_fall = components.generate_df_stat(df, p)
                pct_mean.append(stat_pct)
                rise_mean.append(stat_rise)
                fall_mean.append(stat_fall)


            title_list = html.Div(dbc.Row(title_list))
            pct_list = html.Div(dbc.Row(pct_list))
            rise_list = html.Div(dbc.Row(rise_list))
            fall_list = html.Div(dbc.Row(fall_list))

            hist_area = [title_list,
                         html.Div(style={'height': '5px', }),
                         pct_list,
                         html.Div(style={'height': '5px', }),
                         rise_list, fall_list,]

            title_area = components.selection_title(para_dict, para_radioitems)

            stat_area = components.update_stat_div(period, pct_mean, rise_mean, fall_mean)

            return title_area, stat_area, line_chart, hist_area


        return app


import os
import itertools
import multiprocessing as mp
import zlib

from .equity_curves import *
from .signals import *
from .aggregate import *

def multi_process(function, parameters, number_of_core=8):
    pool = mp.Pool(processes=number_of_core)
    pool.map(function, parameters)
    pool.close()


def filename_only(para_combination):
    para_dict = para_combination['para_dict']
    start_date = para_combination['start_date']
    end_date = para_combination['end_date']
    py_filename = para_combination['py_filename']
    start_date_str = datetime.datetime.strptime(start_date, '%Y-%m-%d').strftime("%Y%m%d")
    end_date_str = datetime.datetime.strptime(end_date, '%Y-%m-%d').strftime("%Y%m%d")
    summary_mode = para_combination['summary_mode']
    freq = para_combination['freq']


    save_name = f'file={py_filename}&date={start_date_str}{end_date_str}&freq={freq}&summary_mode={summary_mode}&'

    for key in list(para_dict.keys()):
        para = para_combination[key]
        if key == 'code':
            if str(para).isdigit():
                para = str(para).zfill(5)

        if isinstance(para, float):
            if para.is_integer():
                para = int(para)

        save_name += f'{key}={str(para)}&'

    return save_name


def path_reference_code(save_name):
    reference_code = str(zlib.crc32(bytes(save_name, 'UTF-8')))[:6]
    return reference_code

def generate_filepath(para_combination,folder=''):
    save_name = filename_only(para_combination)

    reference_code = path_reference_code(save_name)
    save_name = reference_code + '_' + save_name[:-1]     # position -1 to avoid '&' at the end

    output_folder = para_combination['output_folder']
    filepath = os.path.join(folder,output_folder, f'{save_name}.csv')

    return filepath


def mp_cal_performance(tuple_data):
    para_combination = tuple_data[0]
    manager_list = tuple_data[1]

    result = cal_performance(para_combination)
    index = para_combination['reference_index']
    manager_list.append((index,result))


def generate_backtest_result(all_para_combination, number_of_core=8):
    manager_list = mp.Manager().list()# To save the result with index number

    cal_performance_list = []
    for para_combination in all_para_combination:
        cal_performance_list.append((para_combination, manager_list))

    pool = mp.Pool(processes=number_of_core)
    pool.map(mp_cal_performance, cal_performance_list)
    pool.close()

    df_backtest_result = pd.DataFrame()
    df_backtest_result['reference_code'] = np.NaN
    df_backtest_result['reference_index'] = np.NaN
    para_keys = all_para_combination[0]['para_dict'].keys()
    for key in para_keys:
        df_backtest_result[key] = np.NaN
    for key in manager_list[0][1].keys():
        df_backtest_result[key] = np.NaN

    for mp_data in manager_list:
        reference_index = mp_data[0]
        result_dict = mp_data[1]
        para_combination = all_para_combination[reference_index]
        save_name = filename_only(para_combination)
        reference_code = path_reference_code(save_name)
        df_backtest_result.at[reference_index, 'reference_code'] = reference_code
        df_backtest_result.at[reference_index, 'reference_index'] = reference_index
        for key in para_keys:
            df_backtest_result.at[reference_index, key] = para_combination[key]
        for key in result_dict:
            df_backtest_result.at[reference_index, key] = result_dict[key]

    # print(df_backtest_result)
    df_backtest_result['reference_index'] = df_backtest_result['reference_index'].astype(int)
    df_backtest_result.to_csv('backtest_result.csv')



def plot_signal_analysis(py_filename, output_folder, start_date, end_date, para_dict, signal_settings):

    app = signals.Signals(py_filename, output_folder, start_date, end_date, para_dict, generate_filepath, signal_settings)

    return app


def plot(mode, all_para_combination={},  subchart_settings={}):
    if mode == 'equity_curves':
        result_df = pd.read_csv('backtest_result.csv', index_col=0)
        app = equity_curves.Plot(all_para_combination, result_df, subchart_settings)
    if mode == 'aggregate':
        app = aggregate.Aggregate()


    return app




def calculate_mdd(df_csv, col):
    df = df_csv.copy()
    roll_max = df[col].cummax()
    daily_drawdown = df[col] / roll_max - 1.0
    max_daily_drawdown = daily_drawdown.cummin()

    return min(list(max_daily_drawdown)), min(list(df[col] - roll_max))


def calculate_win_rate_info(df_csv):
    df = df_csv.copy()
    num_of_trade = list(df['action'] == 'open').count(True)
    num_of_loss = list(df['pnl'] < 0).count(True)
    num_of_win = num_of_trade - num_of_loss

    if num_of_trade > 0:
        win_rate = round(100 * num_of_win / num_of_trade, 2)
        loss_rate = round(100 * num_of_loss / num_of_trade, 2)
    else:
        win_rate = '--'
        loss_rate = '--'

    # win_rate = str(int(round(100 * num_of_win / num_of_trade, 0))) if num_of_trade > 0 else '--'

    return num_of_trade, num_of_loss, num_of_win, win_rate, loss_rate


def calculate_win_rate(df_csv):
    df = df_csv.copy()
    df = df[df['action'].notnull()].reset_index(drop=True)
    df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
    df['pnl'] = df['realized_pnl'].shift(-1)
    df['year'] = pd.DatetimeIndex(df['date']).year
    year_list = list(set(df['year']))
    year_list.sort()

    win_rate_dict = {}
    win_rate_dict['Overall'] = calculate_win_rate_info(df)

    for year in year_list:
        win_rate_dict[year] = calculate_win_rate_info(df.loc[df['year'] == year])

    return win_rate_dict


def calculate_sharp_ratio(df_csv, col):
    df = df_csv.copy()
    holding_period_day = (df.loc[df.index[-1], 'date'] - df.loc[df.index[0], 'date']).days
    net_profit = df.at[df.index[-1], col] - df.at[df.index[0], col]

    initial_capital = df.loc[df.index[0], col]
    equity_value_pct_series = df[col].pct_change()
    equity_value_pct_series = equity_value_pct_series.dropna()

    return_on_capital = net_profit / initial_capital
    annualized_return = (1 + return_on_capital) ** (365 / holding_period_day) - 1
    annualized_std = equity_value_pct_series.std() * math.sqrt(365)

    if annualized_std > 0:
        annualized_sr = annualized_return / annualized_std
    else:
        annualized_sr = 0

    return_on_capital = round(100 * return_on_capital, 2)
    annualized_return = round(100 * annualized_return, 2)
    annualized_std = round(100 * annualized_std, 2)
    annualized_sr = round(annualized_sr, 2)

    return net_profit, holding_period_day, return_on_capital, annualized_return, annualized_std, annualized_sr


def resample_summary_to_daily(para_combination, folder=''):
    # Need to deal with the start date and date

    start_date = para_combination['start_date']
    end_date = para_combination['end_date']
    sectype = para_combination['sectype']
    lot_size = para_combination['lot_size']
    intraday = para_combination['intraday']
    freq = para_combination['freq']
    sec_profile = para_combination['sec_profile']
    margin_req = sec_profile['margin_req']
    multiplier = sec_profile['multiplier']

    # Read backtest data
    save_path = generate_filepath(para_combination=para_combination,folder=folder)
    df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
    df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
    df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
    df_backtest = df_backtest.reset_index(drop=True)  # Reset Index

    trade_pair = zip(*(iter(list(df_backtest.index)),) * 2)  # Group df_backtest by 2
    initial_capital = df_backtest.iloc[-1].equity_value - df_backtest['realized_pnl'].sum()
    last_realized_capital = initial_capital

    # Read source data
    data_folder = para_combination['data_folder']
    code = para_combination['code']
    data_path = os.path.join(folder,data_folder, f'{code}_{freq}.csv')
    df_csv = pd.read_csv(data_path)
    df_daily = pd.DataFrame()  # Dataframe for Daily
    df_daily['open'] = df_csv['open']
    df_daily['high'] = df_csv['high']
    df_daily['low'] = df_csv['low']
    df_daily['close'] = df_csv['close']
    df_daily['volume'] = df_csv['volume']
    df_daily.index = pd.to_datetime(df_csv['datetime'], format='%Y-%m-%d %H:%M:%S')

    if intraday:
        df_daily = df_daily.resample('1D').agg({'open': 'first', 'high': 'max', 'low': 'min',
                                                'close': 'last', 'volume': 'sum',
                                                }).copy()

    df_daily['date'] = df_daily.index
    df_daily = df_daily.loc[(df_daily['date'] >= start_date) & (df_daily['date'] <= end_date)]
    df_daily = df_daily.loc[df_daily['close'].isnull() == False].copy()  # Filter out non trading day
    df_daily['bah'] = df_daily['close'] * (initial_capital / df_daily.iloc[0].close)  # Calculate all bah value
    df_daily.at[df_daily.index[0], 'equity_value'] = initial_capital  # Set the equity value at the beginning
    df_daily['action'] = None
    df_daily['realized_pnl'] = None
    df_daily['unrealized_pnl'] = None
    df_daily['signal_value'] = None

    for i, j in trade_pair:
        open_date = df_backtest.at[i, 'date']
        close_date = df_backtest.at[j, 'date']
        dates = pd.date_range(start=open_date, end=close_date)  # Determine days between open_date and close_date
        now_close = df_daily.at[open_date, 'close']
        # print(i, j)
        # print(dates)
        # print(df_backtest.at[i, 'date'], df_backtest.at[i, 'action'])
        # print(df_backtest.at[j, 'date'], df_backtest.at[j, 'action'])


        if sectype == 'STK':
            num_of_share = lot_size * (last_realized_capital // (now_close * lot_size))
        elif sectype == 'FUT':
            num_of_share = last_realized_capital // margin_req

        open_price = df_backtest.at[i, 'open_price']
        commission = df_backtest.at[j, 'commission']

        df_daily.at[open_date, 'signal_value'] = df_daily.at[open_date, 'bah']  # Mark the open for each trade_pair

        # print(open_price, df_daily.at[open_date, 'bah'])

        # If only one date in the dates (i.e. open and close at the same day), add one more to avoid error
        if open_date == close_date:
            dates = list(dates) + list(dates)
            dates = pd.to_datetime(dates, format='%Y-%m-%d')

        # For the situation that open and close is not the same day. If same, close pnl will replace this one
        df_daily.at[open_date, 'action'] = 'open'
        unrealized_pnl = num_of_share * multiplier * (now_close - open_price) - commission
        unrealized_pnl = round(unrealized_pnl, 3)
        df_daily.at[open_date, 'unrealized_pnl'] = unrealized_pnl
        # print(open_date, 'Open!', open_price, now_close, unrealized_pnl)

        ### record unrealised pnl and realized pnl and the same time
        ### unrealised pnl for the equity value in between
        for date in dates[1:]:
            if date == close_date:  # close position on this date

                # clear the open pnl as open and close same day should not aggregate
                if open_date == close_date:
                    df_daily.at[date, 'unrealized_pnl'] = None

                # unrealized_pnl
                if df_daily.at[date, 'unrealized_pnl']:  # if there is already unrealized_pnl, aggregate
                    df_daily.at[date, 'unrealized_pnl'] = df_daily.at[date, 'unrealized_pnl'] + df_backtest.at[
                        j, 'realized_pnl']
                else:
                    df_daily.at[date, 'unrealized_pnl'] = df_backtest.at[j, 'realized_pnl']

                # realized pnl
                if df_daily.at[date, 'realized_pnl']:  # if there is already realized_pnl, aggregate
                    df_daily.at[date, 'realized_pnl'] = df_daily.at[date, 'realized_pnl'] + df_backtest.at[
                        j, 'realized_pnl']
                else:
                    df_daily.at[date, 'realized_pnl'] = df_backtest.at[j, 'realized_pnl']

                df_daily.at[date, 'action'] = df_backtest.at[j, 'action']
                df_daily.at[date, 'commission'] = df_backtest.at[j, 'commission']
                last_realized_capital = last_realized_capital + df_backtest.at[j, 'realized_pnl']

                # print(date, 'Close!', open_price, df_backtest.at[j, 'close_price'], df_backtest.at[j, 'realized_pnl'])

            else:
                try:
                    now_close = df_daily.at[date, 'close']
                    unrealized_pnl = num_of_share * multiplier * (now_close - open_price) - commission
                    unrealized_pnl = round(unrealized_pnl, 3)
                    if df_daily.at[date, 'unrealized_pnl']:
                        df_daily.at[date, 'unrealized_pnl'] = df_daily.at[date, 'unrealized_pnl'] + unrealized_pnl
                    else:
                        df_daily.at[date, 'unrealized_pnl'] = unrealized_pnl

                    # print(date, 'Not yet close', open_price, now_close, unrealized_pnl)

                except:
                    pass
                    # print(date, 'Not yet close', 'Holiday!')

        # print()

    last_equity_value = initial_capital
    for i in df_daily.index[1:]:
        if df_daily.at[i, 'realized_pnl']:
            df_daily.at[i, 'equity_value'] = last_equity_value + df_daily.at[i, 'realized_pnl']
            last_equity_value = df_daily.at[i, 'equity_value']

        elif df_daily.at[i, 'unrealized_pnl']:
            df_daily.at[i, 'equity_value'] = last_equity_value + df_daily.at[i, 'unrealized_pnl']

    df_daily['equity_value'] = df_daily['equity_value'].fillna(method='ffill')  # Fill na for equity value only

    # df_daily.to_csv('df_daily.csv')

    return df_daily




def cal_performance(para_combination):

    start_date = para_combination['start_date']
    end_date = para_combination['end_date']

    intraday = para_combination['intraday']
    summary_mode = para_combination['summary_mode']

    if (intraday or summary_mode):
        df_daily = resample_summary_to_daily(para_combination=para_combination)
    else:
        save_path = generate_filepath(para_combination=para_combination)
        df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
        df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
        df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
        df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
        df_daily = df_backtest.copy()

    result_dict = {}

    # Determine years at the beginning
    start_date_year = datetime.datetime.strptime(start_date, '%Y-%m-%d').year
    end_date_year = datetime.datetime.strptime(end_date, '%Y-%m-%d').year
    year_list = list(range(start_date_year, end_date_year + 1))
    for y in year_list: result_dict[str(y)] = []

    # df = pd.read_csv(save_path)
    df = df_daily.copy()
    df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')

    # Strategy Performance
    net_profit, holding_period_day, return_on_capital, annualized_return, annualized_std, annualized_sr = calculate_sharp_ratio(df, 'equity_value')
    mdd_pct, mdd_dollar = calculate_mdd(df, 'equity_value')
    mdd_pct = mdd_pct * -100
    mdd_dollar = mdd_dollar * -1

    # Win Rate (need to use df_backtest directly)
    save_path = generate_filepath(para_combination=para_combination)
    df_backtest = pd.read_csv(save_path, index_col=0)
    win_rate_dict = calculate_win_rate(df_backtest)
    num_of_trade, num_of_loss, num_of_win, win_rate, loss_rate = win_rate_dict['Overall']
    # print('net_profit',net_profit)

    # BaH Performance
    bah_net_return, holding_period_day, bah_return, \
    bah_annualized_return, bah_annualized_std, bah_annualized_sr = calculate_sharp_ratio(df, 'close')
    initial_capital = df.loc[df.index[0], 'equity_value']
    df['bah_equity_curve'] = df['close'] * initial_capital // df.loc[df.index[0], 'close']
    bah_mdd_pct, bah_mdd_dollar = calculate_mdd(df, 'bah_equity_curve')
    bah_mdd_pct = bah_mdd_pct * -100
    bah_mdd_dollar = bah_mdd_dollar * -1

    total_commission = df['commission'].sum()

    result_dict['holding_period_day'] = holding_period_day
    result_dict['total_commission'] = total_commission

    result_dict['net_profit'] = net_profit
    result_dict['return_on_capital'] = return_on_capital
    result_dict['annualized_return'] = annualized_return
    result_dict['annualized_std'] = annualized_std
    result_dict['annualized_sr'] = annualized_sr
    result_dict['mdd_dollar'] = mdd_dollar
    result_dict['mdd_pct'] = mdd_pct
    result_dict['num_of_trade'] = num_of_trade
    result_dict['win_rate'] = win_rate
    result_dict['loss_rate'] = loss_rate
    if mdd_dollar == 0:
        result_dict['net_profit_to_mdd'] = np.inf
    else:
        result_dict['net_profit_to_mdd'] = net_profit / mdd_dollar

    result_dict['bah_return'] = bah_return
    result_dict['bah_annualized_return'] = bah_annualized_return
    result_dict['bah_annualized_std'] = bah_annualized_std
    result_dict['bah_annualized_sr'] = bah_annualized_sr
    result_dict['bah_mdd_dollar'] = bah_mdd_dollar
    result_dict['bah_mdd_pct'] = bah_mdd_pct

    # Cov
    df3 = df[df['action'] == 'open']
    df3 = df3.set_index('date')
    signal_year_count = df3.groupby(lambda x: x.year).size()

    signal_year_std = np.std(signal_year_count)
    signal_year_mean = np.mean(signal_year_count)
    cov = round(signal_year_std / signal_year_mean, 3)

    result_dict['cov'] = cov

    # Win rate by year
    for year in year_list:
        try:
            result_dict[str(year)] = win_rate_dict[year][0]
            result_dict[f'{year}_win_rate'] = win_rate_dict[year][3]

        except:
            result_dict[str(year)] = 0
            result_dict[f'{year}_win_rate'] = '--'

    result_dict['return_to_bah'] = return_on_capital - bah_return

    return result_dict


def cal_performance_old(save_path, start_date, end_date):
    result_dict = {}

    start_date_year = datetime.datetime.strptime(start_date, '%Y-%m-%d').year
    end_date_year = datetime.datetime.strptime(end_date, '%Y-%m-%d').year
    year_list = list(range(start_date_year, end_date_year + 1))
    for y in year_list: result_dict[str(y)] = []

    df = pd.read_csv(save_path)

    df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')

    net_profit = df.loc[df.index[-1], 'net_profit']
    mdd_dollar = df.loc[df.index[-1], 'mdd_dollar']
    mdd_pct = df.loc[df.index[-1], 'mdd_pct']

    signal = df['action'].value_counts()
    df2 = df[df['action'].notnull()].reset_index(drop=True)

    holding_period_day = (df.loc[df.index[-1], 'date'] - df.loc[df.index[0], 'date']).days

    num_of_trade = signal['open'] if 'open' in signal else 0
    num_of_loss = len(df2[df2['realized_pnl'] < 0])
    num_of_win = num_of_trade - num_of_loss

    if num_of_trade > 0:
        win_rate = round(100 * num_of_win / num_of_trade, 2)
        loss_rate = round(100 * num_of_loss / num_of_trade, 2)
    else:
        win_rate = 0
        loss_rate = 0

    initial_capital = df.loc[df.index[0], 'equity_value']
    equity_value_pct_series = df['equity_value'].pct_change()
    equity_value_pct_series = equity_value_pct_series.dropna()

    return_on_capital = net_profit / initial_capital
    annualized_return = (1 + return_on_capital) ** (365 / holding_period_day) - 1
    annualized_std = equity_value_pct_series.std() * math.sqrt(365)

    if annualized_std > 0:
        annualized_sr = annualized_return / annualized_std
    else:
        annualized_sr = 0

    return_on_capital = round(100 * return_on_capital, 2)
    annualized_return = round(100 * annualized_return, 2)
    annualized_std = round(100 * annualized_std, 2)
    annualized_sr = round(annualized_sr, 2)

    bah_return = df.loc[df.index[-1], 'close'] / df.loc[df.index[0], 'close'] - 1
    bah_annualized_return = (1 + bah_return) ** (365 / holding_period_day) - 1
    bah_annualized_std = df['pct_change'].std() * math.sqrt(365)

    if bah_annualized_std > 0:
        bah_annualized_sr = bah_annualized_return / bah_annualized_std
    else:
        bah_annualized_sr = 0

    bah_return = round(100 * bah_return, 2)
    bah_annualized_return = round(100 * bah_annualized_return, 2)
    bah_annualized_std = round(100 * bah_annualized_std, 2)
    bah_annualized_sr = round(bah_annualized_sr, 2)

    df['bah_equity_curve'] = df['close'] * initial_capital // df.loc[df.index[0], 'close']
    df['bah_dd_dollar'] = df['bah_equity_curve'].expanding().max() - df['bah_equity_curve']
    df['bah_dd_pct'] = df['bah_dd_dollar'] / df['bah_equity_curve'].expanding().max()

    bah_mdd_dollar = df['bah_dd_dollar'].max()
    bah_mdd_pct = df['bah_dd_pct'].max()
    bah_mdd_pct = 100 * bah_mdd_pct

    total_commission = df['commission'].sum()

    result_dict['net_profit_to_mdd'] = 0
    result_dict['net_profit'] = net_profit
    result_dict['mdd_dollar'] = mdd_dollar
    result_dict['mdd_pct'] = mdd_pct
    result_dict['num_of_trade'] = num_of_trade
    result_dict['win_rate'] = win_rate
    result_dict['loss_rate'] = loss_rate

    result_dict['holding_period_day'] = holding_period_day
    result_dict['return_on_capital'] = return_on_capital
    result_dict['annualized_return'] = annualized_return
    result_dict['annualized_std'] = annualized_std
    result_dict['annualized_sr'] = annualized_sr

    result_dict['bah_return'] = bah_return
    result_dict['bah_annualized_return'] = bah_annualized_return
    result_dict['bah_annualized_std'] = bah_annualized_std
    result_dict['bah_annualized_sr'] = bah_annualized_sr

    result_dict['bah_mdd_dollar'] = bah_mdd_dollar
    result_dict['bah_mdd_pct'] = bah_mdd_pct

    result_dict['total_commission'] = total_commission

    if mdd_dollar == 0:
        result_dict['net_profit_to_mdd'] = np.inf
    else:
        result_dict['net_profit_to_mdd'] = net_profit / mdd_dollar

    df3 = df[df['action'] == 'open']
    df3 = df3.set_index('date')
    current_year = datetime.datetime.now().year
    signal_year_count = df3.groupby(lambda x: x.year).size()

    signal_year_std = np.std(signal_year_count)
    signal_year_mean = np.mean(signal_year_count)
    cov = round(signal_year_std / signal_year_mean, 3)

    result_dict['cov'] = cov

    for y in year_list:
        try:
            result_dict[str(y)] = signal_year_count[y]
        except:
            result_dict[str(y)] = 0


    result_dict['return_to_bah'] = return_on_capital - bah_return


    # Singal Win Rate By Year
    df_all_year = pd.DataFrame()
    df_signal = df.copy()
    df_all_year['date'] = df_signal['date']
    df_all_year['year'] = pd.DatetimeIndex(df_signal['date']).year
    df_all_year['action'] = df_signal['action']
    df_all_year['realized_pnl'] = df_signal['realized_pnl']
    df_all_year = df_all_year[df_all_year['action'].notnull()].reset_index(drop=True)
    df_all_year['realized_pnl'] = df_all_year['realized_pnl'].shift(-1)

    for current_year in year_list:
        df_year = df_all_year.copy()
        df_year['year'] = pd.DatetimeIndex(df_year['date']).year
        df_year = df_year.loc[df_year['year'] == current_year]

        signal_by_year = df_year['action'].value_counts()
        df_year = df_year[df_year['action'].notnull()].reset_index(drop=True)

        num_of_trade = signal_by_year['open'] if 'open' in signal_by_year else 0
        num_of_loss = len(df_year[df_year['realized_pnl'] < 0])
        num_of_win = num_of_trade - num_of_loss

        win_rate = str(int(round(100 * num_of_win / num_of_trade, 0))) if num_of_trade > 0 else '--'

        result_dict[f'{current_year}_win_rate'] = win_rate

    return result_dict


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="plotguy",
    version="0.9.2",
    author="Plotguy Team",
    author_email="plotguy.info@gmail.com",
    description="Plotguy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[         
        'pandas>=1.5.2',  
        'numpy>=1.24.1',  
        'hkfdb>=2.2',         
        'dash',
        'dash_bootstrap_components',
        'dash_daq',
        'dash_dangerously_set_inner_html',
    ],
    url="https://pypi.org/project/plotguy/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)


import pandas as pd
import numpy as np
import datetime
import math
import json
import ast
import os

from dash import dcc, html    ## pip install dash
import dash_bootstrap_components as dbc
import plotly.express as px
import plotly.graph_objs as go
from plotly.subplots import make_subplots

import plotguy
import dash_dangerously_set_inner_html   # for all Plot Class


class Components:
    chart_bg = '#1f2c56'

    sort_method_dropdown = html.Div(
        dbc.Select(id='sort_method',
                   placeholder="Select Sorting Method",
                   value='Top 20 Net Profit',
                   options=[{'label': 'Top 20 Net Profit', 'value': 'Top 20 Net Profit'},
                            {'label': 'Top 20 Return-BaH % Difference', 'value': 'Top 20 Return to BaH Ratio'},
                            {'label': 'Top 20 Net Profit/MDD Ratio', 'value': 'Top 20 Net Profit to MDD'}, ],
                   style={'border-radius': '5px',   'font-size': '12px', }),
        style={'padding-left': '15px', 'width': '235px'})


    filter_dropdown = html.Div(id='filter_dropdown',
                               children=dbc.Select(id='filter_name',
                                                   placeholder="Select Filter",
                                                   options=[
                                                       {'label': 'Exclude Stock', 'value': 'exclude'},
                                                       {'label': 'Return on Capital >', 'value': 'return_on_capital>'},
                                                       {'label': 'Return on Capital <', 'value': 'return_on_capital<'},
                                                       {'label': 'Annualized Return >', 'value': 'annualized_return>'},
                                                       {'label': 'Annualized Return <', 'value': 'annualized_return<'},
                                                       {'label': 'Return-BaH % Diff. >', 'value': 'return_to_bah>'},
                                                       {'label': 'Return-BaH % Diff. <', 'value': 'return_to_bah<'},
                                                       {'label': 'Sharp Ratio >', 'value': 'annualized_sr>'},
                                                       {'label': 'Sharp Ratio <', 'value': 'annualized_sr<'},
                                                       {'label': 'MDD Percentage >', 'value': 'mdd_pct>'},
                                                       {'label': 'MDD Percentage <', 'value': 'mdd_pct<'},
                                                       {'label': 'Trade Count >', 'value': 'num_of_trade>'},
                                                       {'label': 'Trade Count <', 'value': 'num_of_trade<'},
                                                       {'label': 'COV >', 'value': 'cov>'},
                                                       {'label': 'COV <', 'value': 'cov<'},
                                                       {'label': 'Win Rate >', 'value': 'win_rate>'},
                                                       {'label': 'Win Rate <', 'value': 'win_rate<'},
                                                   ],
                                                   style={'border-radius': '5px', 'font-size': '12px'}),
                               style={'padding-left': '15px', 'width': '175px'})

    filter_dropdown_disabled = html.Div(id='filter_dropdown',
                                        children=dbc.Select(id='filter_name', disabled=True,
                                                            placeholder="Select Filter",
                                                            style={'border-radius': '5px', 'font-size': '12px',
                                                                   'backgroundColor': 'Gray'}),
                                        style={'padding-left': '15px', 'width': '180px'})

    filter_input = dbc.Input(id='filter_input', value=None, size="md", type='text',
                             style={'width': '50px', 'margin-right': '5px', 'border-radius': '3px',
                                      'padding': '6px 5px', 'font-size': '12px', })

    filter_input_disabled = dbc.Input(id='filter_input', value=None, size="md", disabled=True,
                                      style={'width': '50px', 'margin-right': '5px', 'border-radius': '3px',
                                               'padding': '6px 5px', 'font-size': '12px', 'backgroundColor': 'Gray'})

    add_button_style = {'margin-left': '50px', 'width': '150px', 'backgroundColor': 'blue',
                        'border-radius': '5px', 'text-align': 'center', 'cursor': 'pointer',
                        'font-size': '13px', 'height':'30px'}

    add_button_style_disabled = {'margin-left': '50px', 'width': '150px', 'color': 'Silver', 'backgroundColor': 'Gray',
                                 'border-radius': '5px', 'text-align': 'center',
                                 'font-size': '13px', 'height':'30px'}


    def empty_line_chart(self):
        chart_bg = self.chart_bg
        fig_line = px.line()
        fig_line.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, margin=dict(l=85, r=60, t=30, b=40),
                               height=500, font={"color": chart_bg})
        fig_line.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)
        fig_line.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)

        return fig_line


    def update_checkbox_div(self, para_dict, result_df):
        # find the unique values from result_df according to the key from para_dict
        checkbox_values = {}
        for key in para_dict:
            # if not key == 'code':
            unique_values = list(dict.fromkeys(result_df[key].tolist()))
            try:
                unique_values.sort()
            except:
                pass
            checkbox_values[key] = unique_values

        checkbox_div = []
        for i, para_name in enumerate (para_dict):
            # if not para_name == 'code':
            _options = checkbox_values[para_name]
            if _options == [False, True]:
                _checklist = dcc.Checklist([str(tf) for tf in _options], [str(tf) for tf in _options], inline=True,
                                           id={'type': 'para-checklist', 'index': i},
                                           labelStyle={},
                                           inputStyle={'margin-left': '10px', 'margin-right': '3px'})
            else:
                _checklist = dcc.Checklist(_options, _options, inline=True,
                                           id={'type': 'para-checklist', 'index': i},
                                           labelStyle={'font-size': '12px'},
                                           #label_checked_style={"color": "red"},
                                           inputStyle={'margin-left': '10px', 'margin-right': '3px','background-color':'red',
                                                       'vertical-align':'middle','position':'relative',
                                                       'bottom':'.15em'})
            row = html.Div(
                dbc.Row([
                    html.Div(para_name),
                    html.Div(style={'height': '5px'}),
                    html.Div(_checklist),
                    html.Div(style={'height': '5px'}),
                ]), style={'padding': '0px 20px', 'font-size': '12px'})

            checkbox_div.append(row)

        return checkbox_div

    def update_stat_div(self, period, pct_mean, rise_mean, fall_mean):
        div = html.Div([

            dbc.Row([
                 dbc.Col(width=2),
                 dbc.Col(str(period[0]) + ' Days', width=2),
                 dbc.Col(str(period[1]) + ' Days', width=2),
                 dbc.Col(str(period[2]) + ' Days', width=2),
                 dbc.Col(str(period[3]) + ' Days', width=2),
                 dbc.Col(str(period[4]) + ' Days', width=2),
             ]),

            dbc.Row([
                 dbc.Col('pct_change', width=2),
                 dbc.Col(f'{pct_mean[0]}%', width=2),
                 dbc.Col(f'{pct_mean[1]}%', width=2),
                 dbc.Col(f'{pct_mean[2]}%', width=2),
                 dbc.Col(f'{pct_mean[3]}%', width=2),
                 dbc.Col(f'{pct_mean[4]}%', width=2),
             ]),

            dbc.Row([
                 dbc.Col('max_rise', width=2),
                 dbc.Col(f'{rise_mean[0]}%', width=2),
                 dbc.Col(f'{rise_mean[1]}%', width=2),
                 dbc.Col(f'{rise_mean[2]}%', width=2),
                 dbc.Col(f'{rise_mean[3]}%', width=2),
                 dbc.Col(f'{rise_mean[4]}%', width=2),
             ]),

            dbc.Row([
                 dbc.Col('max_fall', width=2),
                 dbc.Col(f'{fall_mean[0]}%', width=2),
                 dbc.Col(f'{fall_mean[1]}%', width=2),
                 dbc.Col(f'{fall_mean[2]}%', width=2),
                 dbc.Col(f'{fall_mean[3]}%', width=2),
                 dbc.Col(f'{fall_mean[4]}%', width=2),
             ]),

        ])

        return div


    def generate_radioitems(self, para_dict):
        radioitems_div = []

        for i, key in enumerate(para_dict):
            options = para_dict[key]

            radioitems_div.append(html.Div(key,style={'color': 'Yellow','font-size': '14px'}))

            radioitems_div.append(dcc.RadioItems(
                id={'type': 'para_radioitems', 'index': i},
                options=[{'label': k,
                          'value': k} for k in options],
                labelStyle={'font-size': '13px'},
                inputStyle={'margin-left': '10px', 'margin-right': '5px'}

            ), )

            radioitems_div.append(html.Div(html.Img(),style={'height':'10px'}))

        return radioitems_div


    def update_performance_matrix(self, start_date,end_date,df,para_dict):
        per_col1 =[]
        per_col2 = []
        per_col_1_1 = []
        per_col_1_2 = []
        per_col_2_1 = [html.Div(html.Img())]
        per_col_2_2 = [html.Div('Selected',style={'text-align': 'center', 'font-weight': 'bold', 'color':'yellow'})]
        per_col_2_3 = [html.Div('BaH',style={'text-align': 'center', 'font-weight': 'bold', 'color':'yellow'})]

        keys = ['num_of_trade',
                'net_profit',
                'net_profit_to_mdd',
                'return_to_bah',
                'win_rate',
                'cov',
                'total_commission',
                'return_on_capital',
                'annualized_return',
                'annualized_std',
                'annualized_sr',
                'mdd_dollar',
                'mdd_pct',
                'bah_return',
                'bah_annualized_return',
                'bah_annualized_std',
                'bah_annualized_sr',
                'bah_mdd_dollar',
                'bah_mdd_pct',
                ]

        try:
            df['net_profit'] = "{:,}".format(int(round(df['net_profit'], 0)))
            if df['net_profit_to_mdd'] == np.inf: df['net_profit_to_mdd'] = 'inf'
            else: df['net_profit_to_mdd'] = round(df['net_profit_to_mdd'], 2)
            df['total_commission'] = "{:,}".format(int(round(df['total_commission'] , 0)))

            df['mdd_dollar'] = "{:,}".format(int(round(df['mdd_dollar'], 0)))
            df['mdd_pct'] = "{:.0%}".format(df['mdd_pct'] / 100)
            df['return_on_capital'] = "{:.0%}".format(df['return_on_capital'] / 100)
            df['annualized_return'] = "{:.0%}".format(df['annualized_return'] / 100)
            df['annualized_std'] = "{:.0%}".format(df['annualized_std'] / 100)

            df['cov'] = round(df['cov'], 2)
            df['win_rate'] = "{:.0%}".format(df['win_rate'] / 100)
            df['return_to_bah'] = "{:.0%}".format(df['return_to_bah'] / 100)

            df['bah_mdd_dollar'] = "{:,}".format(int(round(df['bah_mdd_dollar'], 0)))
            df['bah_mdd_pct'] = "{:.0%}".format(df['bah_mdd_pct'] / 100)
            df['bah_return'] = "{:.0%}".format(df['bah_return'] / 100)
            df['bah_annualized_return'] = "{:.0%}".format(df['bah_annualized_return'] / 100)
            df['bah_annualized_std'] = "{:.0%}".format(df['bah_annualized_std'] / 100)
            # df['bah_annualized_sr'] = round(df['bah_annualized_sr'], 1)
        except Exception as e:
            # print(e)
            pass

        per_col_1_1.append(html.Div('Number of Trade'))
        per_col_1_1.append(html.Div('Net Profit'))
        per_col_1_1.append(html.Div('Net Profit/MDD Ratio'))
        per_col_1_1.append(html.Div('Return-BaH % Diff.'))
        per_col_1_1.append(html.Div('Win Rate'))
        per_col_1_1.append(html.Div('COV'))
        per_col_1_1.append(html.Div('Total Commission'))

        per_col_2_1.append(html.Div('Return on Capital'))
        per_col_2_1.append(html.Div('Ann. Return'))
        per_col_2_1.append(html.Div('Ann. Std'))
        per_col_2_1.append(html.Div('Ann. Sharp Ratio'))
        per_col_2_1.append(html.Div('MDD Dollar'))
        per_col_2_1.append(html.Div('MDD Percentage'))

        for i in range(7):
            per_col_1_2.append(html.Div(df[keys[i]], style={'text-align': 'center'}))
        for i in range(7, 13):
            per_col_2_2.append(html.Div(str(df[keys[i]]), style={'text-align': 'center'}))
        for i in range(13, 19):
            per_col_2_3.append(html.Div(str(df[keys[i]]), style={'text-align': 'center'}))

        per_col1.append(dbc.Row([dbc.Col(html.Div(per_col_1_1), width=6),
                                 dbc.Col(per_col_1_2, style={'padding': '0'}, width=6)]))
        per_col2.append(dbc.Row([dbc.Col(html.Div(per_col_2_1), width=6),
                                 dbc.Col(per_col_2_2, style={'padding': '0'}, width=3),
                                 dbc.Col(per_col_2_3, style={'padding': '0'}, width=3)
                                 ]))

        start_date_year = datetime.datetime.strptime(start_date, '%Y-%m-%d').year
        end_date_year = datetime.datetime.strptime(end_date, '%Y-%m-%d').year
        year_list = list(range(start_date_year, end_date_year + 1))

        year_col1 = [dbc.Row([dbc.Col(style={'margin-left':'15px'}, width=3),
                              dbc.Col('Year Count', style={'text-align': 'center', 'font-weight': 'bold', 'color':'yellow'},width=4),
                              dbc.Col('Win Rate', style={'text-align': 'center', 'font-weight': 'bold', 'color':'yellow'},width=4)
                              ], style={'font-size': '11px'})]


        for i in range(len(year_list)):
            try:
                win_rate = "{:.0%}".format( int(df[f'{year_list[i]}_win_rate'])/100 )
            except:
                win_rate = '-----'

            year_col1.append(dbc.Row([dbc.Col(year_list[i],style={'margin-left':'15px'} ,width=3),
                                      dbc.Col(df[str(year_list[i])], style={'text-align': 'center'} , width=4),
                                      dbc.Col(win_rate, style={'text-align': 'center'} , width=4)]))
        # Selected Equity Curve
        title = []
        for key in para_dict:
            title.append(dbc.Row(
                [dbc.Col(key,width=6),
                dbc.Col(f'{df[key]}',style={ 'text-align': 'center'},width=6)]))



        matrix_div = html.Div([
            html.Div(style={'height': '8px', }),

            html.Div([
                html.Div('Selected Curve', style={'color': 'Cyan',
                                                  'display': 'inline',
                                                  'font-size': '15px'}),
                html.Div('', id='save_status',
                         style={'color': 'Cyan',
                                'display': 'inline',
                                'padding-left': '5px',
                                'font-size': '15px'}),
            ]),

            dbc.Row(html.Div(title), style={'font-size': '11px','padding-left':'1px'}),

            html.Div(style={'height': '15px', }),

            html.Div('Performance', style={'color': 'Cyan', 'font-size': '15px'}),
            html.Div(children=per_col1, style={'font-size': '11px','padding-left':'1px'}),
            html.Div(style={'height': '15px', }),

            html.Div('Comparison', style={'color': 'Cyan', 'font-size': '15px'}),
            html.Div(children=per_col2, style={'font-size': '11px','padding-right': '10px','padding-left':'1px'}),
            html.Div(style={'height': '15px', }),

            html.Div('Performance by Year', style={'color': 'Cyan', 'font-size': '15px'}),
            html.Div(style={'height': '5px', }),
            html.Div(children=year_col1, style={'font-size': '11px'}),
            html.Div(style={'height': '20px'}),

            html.Div([
                html.Div(style={'height': '5px'}),
                html.Div('', id='save_string'),
                html.Div(style={'height': '5px'}),
            ], id='save_button'),

        ],style={'padding': '0px',})

        return matrix_div


    filter_options = {
        'num_of_trade':'Trade Count',
        'return_on_capital': 'Return on Capital',
        'annualized_return': 'Annualized Return',
        'annualized_sr': 'Sharp Ratio',
        'mdd_pct':'MDD Percentage',
        'cov':'COV',
        'win_rate':'Win Rate',
        'return_to_bah': 'Return/BnH % Diff.',
        'exclude': 'Exclude',
        }
    def update_filter_div(self, filter_list):
        filter_button = []
        for i, element in enumerate(filter_list):
            element = filter_list[i]
            filter_full = []
            filter_full.append(html.Div(self. filter_options[element[0]], style={'margin-right': '15px', 'display': 'inline'}))
            filter_full.append(html.Div(element[1], style={'margin-right': '15px', 'display': 'inline'}))
            filter_full.append(html.Div(element[2], style={'margin-right': '15px', 'display': 'inline'}))
            filter_button.append(dbc.Row([
                dbc.Col(html.Div(filter_full,
                                 style={'font-size': '12px', 'padding': '0px', 'margin': '0px'}), width=10),
                dbc.Col(html.Div(children=html.Div('✗', style={'padding': '0px', 'margin': '0px'}),
                                 id='button_' + str(i), n_clicks=i,
                                 style={'font-size': '12px', 'backgroundColor': 'rgba(0, 0, 0, 0)',
                                        'border': '0px black solid', 'padding': '0px', 'padding-bottom': '10px',
                                        'margin': '0px', 'width': '5px', 'cursor': 'pointer'}), width=2)
            ]))

        for i in range(len(filter_list), 10):
            filter_button.append(html.Div(id='button_' + str(i), n_clicks=i))

        return filter_button


    def sort_method_df(self, sort_method, result_df):
        if sort_method == 'Top 20 Net Profit':
            df_sorted = result_df.sort_values(by='net_profit', ascending=False).head(20).copy()
        elif sort_method == 'Top 20 Net Profit to MDD':
            df_sorted = result_df.sort_values(by='net_profit_to_mdd', ascending=False).head(20).copy()
        elif sort_method == 'Top 20 Return to BaH Ratio':
            df_sorted = result_df.sort_values(by='return_to_bah', ascending=False).head(20).copy()
        else:
            df_sorted = result_df.copy()

        df_sorted = df_sorted.reset_index(drop=True)


        line_colour = []
        for c in range(len(df_sorted)):
            profile = c % 6
            degree = (c // 6) / math.ceil(len(df_sorted) / 6)
            line_colour.append(self.assign_colour(profile, degree))
        df_sorted['line_colour'] = line_colour

        return df_sorted


    def assign_colour(self, profile, degree):
        if profile == 0:    rgb = (0, int(252 - 252 * degree), 252)
        elif profile == 1:  rgb = (int(252 - 252 * degree), 252, 0)
        elif profile == 2:  rgb = (252, 0, int(252 - 252 * degree))
        elif profile == 3:  rgb = (0, 252, int(252 * degree))
        elif profile == 4:  rgb = (252, int(252 * degree), 0)
        elif profile == 5:  rgb = (int(252 * degree), 0, 252)
        return 'rgb' + str(rgb)

    def generate_chart_3(self, para_combination, chart3_start, chart3_end, freq):

        data_folder = para_combination['data_folder']
        code = para_combination['code']
        data_path = os.path.join(data_folder, code + '.csv')
        df = pd.read_csv(data_path)

        df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
        df['datetime'] = pd.to_datetime(df['datetime'], format='%Y-%m-%d %H:%M:%S')
        df = df.loc[(df['date'] >= chart3_start) & (df['date'] <= chart3_end)]

        # grab first and last observations from df.date and make a continuous date range from that
        dt_all = pd.date_range(start=df['datetime'].iloc[0], end=df['datetime'].iloc[-1], freq=freq)
        # check which dates from your source that also accur in the continuous date range
        dt_obs = [d.strftime("%Y-%m-%d %H:%M:%S") for d in df['datetime']]
        # isolate missing timestamps
        dt_breaks = [d for d in dt_all.strftime("%Y-%m-%d %H:%M:%S").tolist() if not d in dt_obs]

        save_path = plotguy.generate_filepath(para_combination)
        df_signal_list = pd.read_csv(save_path)
        df_signal_list['date'] = pd.to_datetime(df_signal_list['date'], format='%Y-%m-%d')
        df_signal_list['datetime'] = pd.to_datetime(df_signal_list['datetime'], format='%Y-%m-%d %H:%M:%S')
        df_signal_list = df_signal_list.loc[
            (df_signal_list['date'] >= chart3_start) & (df_signal_list['date'] <= chart3_end)]

        df_open = df_signal_list.loc[df_signal_list['action'] == 'open']
        df_profit = df_signal_list.loc[df_signal_list['action'] == 'profit_target']
        df_stop_loss = df_signal_list.loc[df_signal_list['action'] == 'stop_loss']
        df_close = df_signal_list.loc[df_signal_list['action'] == 'close_logic']

        fig = go.Figure(data=[go.Candlestick(x=df['datetime'],
                                             open=df['open'],
                                             high=df['high'],
                                             low=df['low'],
                                             close=df['close'],
                                             increasing_line_color='white')
                              ])

        fig.update_layout(title = {'text': 'Candlestick Chart' , 'font': {'size': 12 }  })

        fig.update_xaxes(rangebreaks=[dict(dvalue=15 * 60 * 1000, values=dt_breaks)])

        fig.add_trace(go.Scatter(mode='markers',
                                 x=df_open['datetime'], y=df_open['close'],  # visible='legendonly',
                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
                                             line=dict(color='yellow', width=2.5)), name='Open'))
        fig.add_trace(go.Scatter(mode='markers',
                                 x=df_profit['datetime'], y=df_profit['close'],  # visible='legendonly',
                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
                                             line=dict(color='Cyan', width=2.5)), name='Profit Target'))
        fig.add_trace(go.Scatter(mode='markers',
                                 x=df_stop_loss['datetime'], y=df_stop_loss['close'],  # visible='legendonly',
                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
                                             line=dict(color='Cyan', width=2.5)), name='Stop Loss'))
        fig.add_trace(go.Scatter(mode='markers',
                                 x=df_close['datetime'], y=df_close['close'],  # visible='legendonly',
                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
                                             line=dict(color='Cyan', width=2.5)), name='Close Logic'))

        fig.update_layout(xaxis_rangeslider_visible=False)

        chart_bg = '#1f2c56'
        fig.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, height=500,
                          margin=dict(l=85, r=25, t=60, b=0),
                          showlegend=False,
                          font={"color": "white", 'size': 10.5}, yaxis={'title': 'Equity'},
                          xaxis={'title': ''}
                          )
        fig.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)
        fig.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)




        return fig


    def generate_chart_1(self, graph_df, all_para_combination, intraday, summary_mode):
        py_filename = all_para_combination[0]['py_filename']
        para_dict = all_para_combination[0]['para_dict']
        para_key_list = list(para_dict)
        df_all_para_combination = pd.DataFrame(all_para_combination)

        fig_line = px.line()
        fig_line.update_layout(title = {'text': py_filename})
        fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
        fig_line.update_yaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
        fig_line.update_layout(plot_bgcolor=self.chart_bg, paper_bgcolor=self.chart_bg, height=500,
                               margin=dict(l=85, r=25, t=60, b=0),
                               showlegend=False,
                               font={"color": "white", 'size': 10.5}, yaxis={'title': 'Equity'},
                               xaxis={'title': ''}
                               )

        for i in graph_df.index:
            # print(df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict())
            para_combination = {}
            hovertemplate = "%{x}<br>"
            for key in para_key_list:
                para_combination[key] = graph_df.iloc[i][key]
                hovertemplate = hovertemplate + \
                                key + " : " + str(graph_df.iloc[i][key]) + "<br>"
            hovertemplate = hovertemplate + "<br>"
            hovertemplate = hovertemplate + "Return-BaH % Diff. : " + str(
                round(graph_df.iloc[i]['return_to_bah'], 2)) + "<br>"
            hovertemplate = hovertemplate + "Net Profit to MDD: " + str(
                round(graph_df.iloc[i]['net_profit_to_mdd'], 2)) + "<br>"
            hovertemplate = hovertemplate + "Sharp Ratio : " + str(graph_df.iloc[i]['annualized_sr']) + "<br>"
            hovertemplate = hovertemplate + "MDD Percentage : " + "{:.0%}".format(
                graph_df.iloc[i]['mdd_pct'] / 100) + "<br>"
            hovertemplate = hovertemplate + "Trade Count : " + str(graph_df.iloc[i]['num_of_trade']) + "<br>"
            hovertemplate = hovertemplate + "COV : " + str(round(graph_df.iloc[i]['cov'], 2)) + "<br>"
            hovertemplate = hovertemplate + "Win Rate : " + "{:.0%}".format(
                graph_df.iloc[i]['win_rate'] / 100) + "<br>"
            hovertemplate = hovertemplate + "<br>"

            # para_combination for save_path generation
            reference_index = graph_df.iloc[i].reference_index
            para_combination = df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict('records')[0]

            line_colour = graph_df.loc[i].line_colour



            # df = pd.read_csv(save_path)
            # df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')

            if (intraday or summary_mode):
                df_daily = plotguy.resample_summary_to_daily(para_combination=para_combination)
            else:
                save_path = plotguy.generate_filepath(para_combination)
                df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
                # df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
                # df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
                # df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
                df_daily = df_backtest.copy()

            start_date = para_combination['start_date']
            end_date = para_combination['end_date']
            df_daily['date'] = pd.to_datetime(df_daily['date'], format='%Y-%m-%d')
            df_daily = df_daily.loc[(df_daily['date'] >= start_date) & (df_daily['date'] <= end_date)]
            df_daily = df_daily.reset_index(drop=True)  # Reset Index

            hovertemplate = hovertemplate + "Equity : %{y:,.0f}"

            fig_line.add_trace(go.Scatter(mode='lines', hovertemplate=hovertemplate,
                                          x=df_daily['date'], y=df_daily['equity_value'],
                                          line=dict(color=line_colour, width=1.5), name=''), )
        return fig_line


    def prepare_df_chart(self, df):     # Chart Data for Chart 2
        initial_value = df.iloc[0].equity_value

        df_chart = df.copy()
        df_chart['date'] = pd.to_datetime(df_chart['date'], format='%Y-%m-%d')
        df_chart['bah'] = df['close'] * (initial_value / df_chart.iloc[0].close)

        _open = []
        _stop_loss = []
        _close_logic = []
        _profit_target = []

        for i in range(len(list(df['action']))):
            element = list(df['action'])[i]
            if element == 'open':
                _open.append(df_chart.iloc[i].bah)
                _stop_loss.append(None)
                _close_logic.append(None)
                _profit_target.append(None)
            elif element == 'stop_loss':
                _open.append(None)
                _stop_loss.append(df_chart.iloc[i].bah)
                _close_logic.append(None)
                _profit_target.append(None)
            elif element == 'close_logic':
                _open.append(None)
                _stop_loss.append(None)
                _close_logic.append(df_chart.iloc[i].bah)
                _profit_target.append(None)
            elif element == 'profit_target':
                _open.append(None)
                _stop_loss.append(None)
                _close_logic.append(None)
                _profit_target.append(df_chart.iloc[i].bah)
            else:
                _open.append(None)
                _stop_loss.append(None)
                _close_logic.append(None)
                _profit_target.append(None)

        df_chart['open'] = _open
        df_chart['stop_loss'] = _stop_loss
        df_chart['close_logic'] = _close_logic
        df_chart['profit_target'] = _profit_target

        return df_chart

    def generate_chart_2_summary(self, para_combination, line_colour):
        df_daily = plotguy.resample_summary_to_daily(para_combination=para_combination)

        title = ''
        para_dict = para_combination['para_dict']
        for key in para_dict:
            title = title + f'{key}:{para_combination[key]} '

        start_date = para_combination['start_date']
        end_date = para_combination['end_date']
        df_daily['date'] = pd.to_datetime(df_daily['date'], format='%Y-%m-%d')
        df_daily = df_daily.loc[(df_daily['date'] >= start_date) & (df_daily['date'] <= end_date)]
        df_daily = df_daily.reset_index(drop=True)  # Reset Index
        df_chart = df_daily.copy()

        row_height = [0.85, 0.15]
        fig_line = make_subplots(rows=2, cols=1,
                                 row_heights=row_height, shared_xaxes=True)


        # fig_line = px.line()
        fig_line.update_layout(title={'text': title, 'font': {'size': 12}})
        fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
                                      x=df_chart['date'], y=df_chart['equity_value'],
                                      line=dict(color=line_colour, width=1), name='Strategy Equity'))
        fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
                                      x=df_chart['date'], y=df_chart['bah'],
                                      line=dict(color='Grey', width=1), name='BnH Equity'))

        hovertemplate = "Date : %{x}"
        fig_line.add_trace(go.Scatter(mode='markers', hovertemplate=hovertemplate,
                                      x=df_chart['date'], y=df_chart['signal_value'], # visible='legendonly',
                                      marker=dict(color='rgba(0, 0, 0, 0)', size=9,
                                                  line=dict(color='yellow', width=2.5)), name='Open'))

        chart_bg = '#1f2c56'
        fig_line.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, height=500,
                               margin=dict(l=85, r=25, t=60, b=0),
                               # showlegend=False,
                               font={"color": "white", 'size': 10.5}, yaxis={'title': 'Equity'},
                               xaxis={'title': ''}
                               )
        fig_line.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)
        fig_line.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)

        fig = self.generate_subchart(df_chart, 'volume', 'line', 'yellow')
        fig_line.add_trace(fig, row=2, col=1)


        return df_daily, fig_line




        pass


    def generate_chart_2_full(self, para_combination, line_colour, settings):
        save_path = plotguy.generate_filepath(para_combination)
        df_csv = pd.read_csv(save_path)
        df_chart = self.prepare_df_chart(df_csv)

        df_signal = df_chart.copy()
        df_signal = df_signal.dropna(subset=['action'])
        df_signal = df_signal.reset_index()
        signal_open_date = []
        signal_open_close = []
        signal_close_date = []
        signal_close_close = []
        signal_close_reason = []

        for i, row in df_signal.iterrows():
            if (row['action'] == 'open') and not ( (i+1) == len(df_signal)):
                signal_open_date.append(df_signal.iloc[i]['date'])
                signal_open_close.append(df_signal.iloc[i]['close'])
                signal_close_date.append(df_signal.iloc[i + 1]['date'])
                signal_close_close.append(df_signal.iloc[i + 1]['close'])
                signal_close_reason.append(df_signal.iloc[i + 1]['action'])
            else:
                signal_open_date.append(df_signal.iloc[i - 1]['date'])
                signal_open_close.append(df_signal.iloc[i - 1]['close'])
                signal_close_date.append(df_signal.iloc[i]['date'])
                signal_close_close.append(df_signal.iloc[i]['close'])
                signal_close_reason.append(df_signal.iloc[i]['action'])

        df_signal['signal_open_date'] = signal_open_date
        df_signal['signal_open_close'] = signal_open_close
        df_signal['signal_close_date'] = signal_close_date
        df_signal['signal_close_close'] = signal_close_close
        df_signal['close_reason'] = signal_close_reason
        df_signal = df_signal.set_index('date')

        open_date = []
        open_close = []
        close_date = []
        close_close = []
        close_reason = []
        for i, row in df_chart.iterrows():
            try:
                open_date.append(df_signal.loc[row['date']].signal_open_date.strftime("%Y-%m-%d"))
            except:
                open_date.append(None)
            try:
                open_close.append(df_signal.loc[row['date']].signal_open_close)
            except:
                open_close.append(None)
            try:
                close_date.append(df_signal.loc[row['date']].signal_close_date.strftime("%Y-%m-%d"))
            except:
                close_date.append(None)
            try:
                close_close.append(df_signal.loc[row['date']].signal_close_close)
            except:
                close_close.append(None)
            try:
                close_reason.append(df_signal.loc[row['date']].close_reason)
            except:
                close_reason.append(None)


        df_signal_final = pd.DataFrame()
        df_signal_final['open_date'] = open_date
        df_signal_final['open_close'] = open_close
        df_signal_final['close_date'] = close_date
        df_signal_final['close_close'] = close_close
        df_signal_final['close_reason'] = close_reason
        df_signal_final['pctchange'] = ((df_signal_final['close_close'] - df_signal_final['open_close']) /
                                        df_signal_final['open_close']) * 100
        df_signal_final.open_close = df_signal_final.open_close.round(2)
        df_signal_final.close_close = df_signal_final.close_close.round(2)
        df_signal_final.pctchange = df_signal_final.pctchange.round(2)

        # print(df_signal_final)

        hover_template = "<br>".join([
            "Close Reason: %{customdata[4]}",
            "Open Date: %{customdata[0]}",
            "Close Date: %{customdata[2]}",
            "Open Close: %{customdata[1]}",
            "Close Close: %{customdata[3]}",
            "Pct Change: %{customdata[5]}%",
        ])


        title = ''
        para_dict = para_combination['para_dict']
        for key in para_dict:
            title = title + f'{key}:{para_combination[key]} '

        # Count number of subchart
        subchart_count = 0
        try:
            fig = settings['subchart_1']
            subchart_count += 1
        except Exception as e: pass
        try:
            fig = settings['subchart_2']
            subchart_count += 1
        except Exception as e: pass

        if subchart_count == 0: row_height = [1]
        if subchart_count == 1: row_height = [0.85,0.15]
        if subchart_count == 2: row_height = [0.7,0.15,0.15]

        fig_line = make_subplots(rows=subchart_count+1, cols=1,
                                 row_heights=row_height,shared_xaxes=True)

        fig_line.update_layout(title = {'text': title , 'font': {'size': 12 }  })
        fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
        fig_line.update_yaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
        fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
                                      x=df_chart['date'], y=df_chart['equity_value'],
                                      line=dict(color=line_colour, width=1), name='Strategy Equity'), row=1, col=1)
        fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
                                      x=df_chart['date'], y=df_chart['bah'],
                                      line=dict(color='Grey', width=1), name='BnH Equity'),row=1, col=1 )
        fig_line.update_layout(plot_bgcolor=self.chart_bg, paper_bgcolor=self.chart_bg, height=500,
                               margin=dict(l=85, r=25, t=35, b=20),
                               font={"color": "white", 'size': 9}, yaxis={'title': 'Equity'},
                               xaxis={'title': ''}
                               )
        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                      x=df_chart['date'], y=df_chart['open'], # visible='legendonly',
                                      marker=dict(color='rgba(0, 0, 0, 0)', size=9,
                                                  line=dict(color='yellow', width=2.5)), name='open'), row=1, col=1)
        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                      x=df_chart['date'], y=df_chart['close_logic'], visible='legendonly',
                                      marker=dict(color='rgba(0, 0, 0, 0)', size=9,
                                                  line=dict(color='green', width=2.5)), name='close_logic'), row=1, col=1)
        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                      x=df_chart['date'], y=df_chart['profit_target'], visible='legendonly',
                                      marker=dict(color='rgba(0, 0, 0, 0)', size=9,
                                                  line=dict(color='red', width=2.5)), name='profit_target'), row=1, col=1)
        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                      x=df_chart['date'], y=df_chart['stop_loss'], visible='legendonly',
                                      marker=dict(color='rgba(0, 0, 0, 0)', size=9,
                                                  line=dict(color='Cyan', width=2.5)), name='stop_loss'), row=1, col=1)


        subchart_count = 2
        try:
            fig = self.generate_subchart(df_chart, settings['subchart_1'][0], settings['subchart_1'][1], 'yellow')
            fig_line.add_trace(fig, row=subchart_count, col=1)
            subchart_count += 1
        except Exception as e:
            # print(e)
            pass
        try:
            fig = self.generate_subchart(df_chart, settings['subchart_2'][0], settings['subchart_2'][1], '#FF01FE')
            fig_line.add_trace(fig, row=subchart_count, col=1)
        except Exception as e:
            # print(e)
            pass



        return df_csv, fig_line


    def generate_subchart(self,df_chart, element, line_type, line_color):
        if line_type == 'bar':
            fig = go.Bar(hoverinfo='skip', x=df_chart['date'], y=df_chart[element], showlegend=True,
                           marker_color=line_color, marker_line_color=line_color, name=element)
        else:
            fig = go.Scatter(mode='lines', hoverinfo='skip', x=df_chart['date'], y=df_chart[element], showlegend=True,
                           line=dict(color=line_color, width=1.5), name=element)
        return fig


    def generate_histogram(self, df, period, mode):
        chart_bg = self.chart_bg
        col_pct = 'pct_change_' + str(period)
        col_rise = 'max_rise_' + str(period)
        col_fall = 'max_fall_' + str(period)
        df_his = df.copy()
        df_his[col_pct] = df_his['close'].pct_change(period)
        df_his[col_pct] = df_his[col_pct].shift(-1 * period)
        df_his[col_pct] = df_his[col_pct] * 100
        df_his[col_pct] = df_his[col_pct].map(lambda x: round(x, 2))
        df_his[col_rise] = (df_his['high'].rolling(period).max().shift(-1 * (period)) / df_his['close']) - 1
        df_his[col_fall] = (df_his['low'].rolling(period).min().shift(-1 * (period)) / df_his['close']) - 1
        if mode == 'backtest':
            df_his = df_his[df_his['action'] == 'open']
        else:
            df_his = df_his[df_his['logic'] == 'trade_logic']

        df_his[col_rise] = df_his[col_rise] * 100
        df_his[col_fall] = df_his[col_fall] * 100

        df_his[col_rise] = df_his[col_rise].map(lambda x: round(x, 2))
        df_his[col_fall] = df_his[col_fall].map(lambda x: round(x, 2))

        df_his = df_his[['date'] + [col_pct, col_rise, col_fall]]

        margin = dict(l=5, r=5, t=5, b=15)
        h = 70
        w = 120
        f = {"color": "white", 'size': 8}

        fig_pct = go.Figure()
        fig_pct.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, height=h, width=w, margin=margin,
                              font=f, bargap=0.1)
        fig_pct.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor='#1f2c56', )
        fig_pct.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor='#626567', )
        fig_pct.add_trace(go.Histogram(x=df_his[col_pct], marker_color='Cyan'))

        fig_rise = go.Figure()
        fig_rise.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, height=h, width=w, margin=margin,
                               font=f, bargap=0.1)
        fig_rise.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor='#1f2c56', )
        fig_rise.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor='#626567', )
        fig_rise.add_trace(go.Histogram(x=df_his[col_rise], marker_color='Yellow'))

        fig_fall = go.Figure()
        fig_fall.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, height=h, width=w, margin=margin,
                               font=f, bargap=0.1)
        fig_fall.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor='#1f2c56', )
        fig_fall.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor='#626567', )
        fig_fall.add_trace(go.Histogram(x=df_his[col_fall], marker_color='Fuchsia'))

        return fig_pct, fig_rise, fig_fall


    def path_to_dict(self,path):
        filename = 'file=' + path.split('file=')[-1].split('.csv')[0]
        parameters = {}
        for element in filename.split('&'):
            key, content = element.split('=')
            if key == 'date':
                # parameters['date'] = f'{content[:8]} - {content[8:]}'
                parameters['startdate'], parameters['enddate'] = content[:8], content[8:]
            elif key == 'summary_mode':
                pass
            else:
                parameters[key] = content

        return parameters


    def generate_df_stat(self, df, period):
        col_pct = 'pct_change_' + str(period)
        col_rise = 'max_rise_' + str(period)
        col_fall = 'max_fall_' + str(period)
        df_his = df.copy()
        df_his[col_pct] = df_his['close'].pct_change(period)
        df_his[col_pct] = df_his[col_pct].shift(-1 * period)
        df_his[col_pct] = df_his[col_pct] * 100
        df_his[col_pct] = df_his[col_pct].map(lambda x: round(x, 2))
        df_his[col_rise] = (df_his['high'].rolling(period).max().shift(-1 * (period)) / df_his['close']) - 1
        df_his[col_fall] = (df_his['low'].rolling(period).min().shift(-1 * (period)) / df_his['close']) - 1

        df_his = df_his[df_his['logic'] == 'trade_logic']

        df_his[col_rise] = df_his[col_rise] * 100
        df_his[col_fall] = df_his[col_fall] * 100

        df_his[col_rise] = df_his[col_rise].map(lambda x: round(x, 2))
        df_his[col_fall] = df_his[col_fall].map(lambda x: round(x, 2))

        return round(df_his[col_pct].mean(), 2), round(df_his[col_rise].mean(), 2), round(df_his[col_fall].mean(), 2)


    def aggregate_df(self):
        folders = []
        for filename in os.listdir():
            if os.path.isfile(f'{filename}/saved_strategies.csv'):
                folders.append(filename)
        folders.sort()

        equity_curves = []
        for folder in folders:
            df_saved = pd.read_csv(f'{folder}/saved_strategies.csv')
            for i, row in df_saved.iterrows():
                equity_path = f'{folder}/{row["path"]}'
                parameters = self.path_to_dict(equity_path)
                dict = {}
                dict['folder'] = folder
                dict['py'] = parameters['file']
                dict['path'] = equity_path
                dict['parameters'] = parameters
                dict['performance'] = {
                    'initial_capital': row['initial'],
                    'net_profit_to_mdd': row['net_profit_to_mdd'],
                    'net_profit': row['net_profit'],
                    'mdd_dollar': row['mdd_dollar'],
                    'mdd_pct': row['mdd_pct'],
                    'num_of_trade': row['num_of_trade'],
                    'num_of_win': row['num_of_win'],
                    'return_on_capital': row['return_on_capital'],
                    'total_commission': row['total_commission'],
                    'sharp_ratio': row['sharp'],
                }
                dict['para_combination'] = ast.literal_eval(row['para_combination'])
                start_date_year = datetime.datetime.strptime(parameters['startdate'], '%Y%m%d').year
                end_date_year = datetime.datetime.strptime(parameters['enddate'], '%Y%m%d').year
                year_list = list(range(start_date_year, end_date_year + 1))
                for year in year_list:
                    dict['performance'][f'{year}'] = row[f'{year}']
                    dict['performance'][f'{year}'] = row[f'{year}']
                    dict['performance'][f'{year}_win_count'] = row[f'{year}_win_count']

                equity_curves.append(dict)

        line_colour = []
        for c in range(len(equity_curves)):
            profile = c % 6
            degree = (c // 6) / math.ceil(len(equity_curves) / 6)
            line_colour.append(self.assign_colour(profile, degree))

        df = pd.DataFrame.from_dict(equity_curves, orient='columns', dtype=None, columns=None)
        df['line_colour'] = line_colour
        df.index += 1

        choices = []
        for i in list(df.index):
            line_height = '120%'

            parameters_text = []
            for key, value in df.iloc[i - 1].parameters.items():
                if not key == 'file':
                    parameters_text.append(html.Div(f'{key} : {value}',style={'line-height': line_height}))

            performance_text = []
            for key, value in df.iloc[i - 1].performance.items():
                if key == 'initial_capital':
                    item = 'Initial Capital'
                    _value = "{:,}".format(int(value))
                    performance_text.append(html.Div(f'{item} : {_value}', style={'line-height': line_height}))

                if key == 'num_of_trade':
                    item = 'Number of Trade'
                    _value = "{:,}".format(int(value))
                    performance_text.append(html.Div(f'{item} : {_value}', style={'line-height': line_height}))

                if key == 'mdd_pct':
                    item = 'MDD Percentage'
                    _value = "{:.0%}".format(value/100)
                    performance_text.append(html.Div(f'{item} : {_value}', style={'line-height': line_height}))

                if key == 'return_on_capital':
                    item = 'Return on Capital'
                    _value = "{:.0%}".format(value/100)
                    performance_text.append(html.Div(f'{item} : {_value}', style={'line-height': line_height}))

                # performance_text.append(html.Div(f'{key} : {value}',style={'line-height': line_height}))

            choices.append({
                "label": html.Div([

                    html.Div('-----------------------------------------------------------------'),

                    dbc.Row([
                        dbc.Col([

                            dbc.Row([
                                dbc.Col([html.Div('Curve', style={'line-height': line_height})],
                                        width=3),
                                dbc.Col([html.Div(str(i).zfill(3), style={'line-height': line_height})],
                                        width=6),
                            ]),

                            html.Div(style={'height': '5px'}),

                            dbc.Row([
                                dbc.Col([html.Div('folder',style={'line-height':line_height})],
                                        width=3),
                                dbc.Col([html.Div(df.iloc[i - 1].folder,style={'line-height':line_height})],
                                        width=6),
                            ]),

                            html.Div(style={'height': '5px'}),

                            dbc.Row([
                                dbc.Col([html.Div('.py',style={'line-height':line_height})],
                                        width=3),
                                dbc.Col([html.Div(df.iloc[i - 1].py,style={'line-height':line_height})],
                                        width=6),
                            ]),

                            html.Div(style={'height': '5px'}),

                            dbc.Row([
                                dbc.Col([html.Div('parameters',style={'line-height':line_height})],
                                        width=3),
                                dbc.Col([html.Div(parameters_text,style={'line-height':line_height})],
                                        width=6),
                            ]),

                            html.Div(style={'height': '5px'}),

                            dbc.Row([
                                dbc.Col([html.Div('performance', style={'line-height': line_height})],
                                        width=3),
                                dbc.Col([html.Div(performance_text, style={'line-height': line_height})],
                                        width=6),
                            ]),
                        ],width=12),

                    ]),

                ],style={"display": "inline-block", 'width':'320px'}

                ),

                "value": i,
            })

        _checklist = html.Div([
            dcc.Checklist(
                options=choices,
                id='curve_checklist',
                value=list(df.index),
                inputStyle={'margin-left': '3px', 'margin-right': '10px',
                               'background-color': 'red',
                               'vertical-align': 'top', 'position': 'relative',
                               'top': '1.6em'}
            )],

        ),

        row = html.Div(
            dbc.Row([
                html.Div(_checklist),
                html.Div(style={'height': '5px'}),
                html.Div('--------------------------------------------------------------------------------',
                         style={'padding-left': '23px'}),
                html.Div(style={'height': '10px', }),
            ]), style={'padding': '0px', 'font-size': '12px'})

        return df, row


    def aggregate_performance(self, total_dict, year_list):
        num_of_trade = "{:,}".format(int(total_dict['num_of_trade']))
        net_profit = "{:,}".format(int(round(total_dict['net_profit'],0)) )
        net_profit_to_mdd = round(total_dict['net_profit_to_mdd'],2)
        win_rate = "{:.0%}".format(total_dict['win_rate'])

        total_commission = "{:,}".format(int(total_dict['total_commission']))
        return_on_capital = "{:.0%}".format(total_dict['return_on_capital'])
        annualized_return = "{:.0%}".format(total_dict['annualized_return'])
        annualized_std = "{:.0%}".format(total_dict['annualized_std'])
        annualized_sr = round(total_dict['annualized_sr'],2)
        mdd_dollar = "{:,}".format(int(total_dict['mdd_dollar']))
        mdd_pct = "{:.0%}".format(total_dict['mdd_pct'])

        year_col = []
        year_count = []
        year_count_col = []
        year_win_rate_col = []
        for year in year_list:
            year_col.append(html.Div(year))
            year_count.append(int(total_dict[f'{year}']))
            year_count_col.append(html.Div(int(total_dict[f'{year}'])))
            rate = total_dict[f'{year}_win_count']/total_dict[f'{year}']
            year_win_rate_col.append(html.Div( "{:.0%}".format(rate) ))

        cov = round(np.std(year_count) / np.mean(year_count), 2)



        div = html.Div([

            html.Div('Performance', style={'color': 'Cyan', 'font-size': '15px'}),

            html.Div(style={'height': '5px'}),

            dbc.Row([
                dbc.Col([html.Div('Number of Trade'),
                         html.Div('Net Profit'),
                         html.Div('Net Profit/MDD Ratio'),
                         html.Div('Win Rate'),
                         html.Div('COV'),
                         html.Div('Total Commission'),
                         html.Div('Return on Capital'),
                         html.Div('Ann. Return'),
                         html.Div('Ann. Std'),
                         html.Div('Ann. Sharp Ratio'),
                         html.Div('MDD Dollar'),
                         html.Div('MDD Percentage')],
                        style={'padding-left':'13px'},
                        width=7),
                dbc.Col([html.Div(num_of_trade),
                         html.Div(net_profit),
                         html.Div(net_profit_to_mdd),
                         html.Div(win_rate),
                         html.Div(cov),
                         html.Div(total_commission),
                         html.Div(return_on_capital),
                         html.Div(annualized_return),
                         html.Div(annualized_std),
                         html.Div(annualized_sr),
                         html.Div(mdd_dollar),
                         html.Div(mdd_pct),
                         ],
                        style={'text-align': 'center',},width=5),
            ]),

            html.Div(style={'height': '10px'}),

            html.Div('Performance by Year', style={'color': 'Cyan', 'font-size': '15px'}),

            html.Div(style={'height': '5px'}),

            dbc.Row([
                dbc.Col(width=3),
                dbc.Col('Year Count', style={'color': 'Yellow', 'font-size': '15px'},width=5),
                dbc.Col('Win Rate', style={'color': 'Yellow', 'font-size': '15px'},width=4),
            ], style={'text-align': 'center'}),

            dbc.Row([
                dbc.Col(year_col, width=3),
                dbc.Col(year_count_col, width=5),
                dbc.Col(year_win_rate_col, width=4),
            ], style={'text-align': 'center'})

        ])

        return div